@@ -1,8 +1,8 @@
 # Vend::Data - Interchange databases
 #
-# $Id: Data.pm,v 2.54.2.1 2006/04/06 20:06:00 jon Exp $
+# $Id: Data.pm,v 2.63 2007/03/30 11:39:44 pajamian Exp $
 # 
-# Copyright (C) 2002-2004 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Data;
 require Exporter;
@@ -44,7 +44,6 @@ item_description
 item_field
 item_price
 item_subtotal
-sql_query
 open_database
 product_category
 product_code_exists_ref
@@ -246,7 +245,6 @@ sub database_field {
 sub database_row {
     my ($db, $key) = @_;
     $db = database_exists_ref($db) or return undef;
-return undef unless defined $db;
     return '' unless $db->test_record($key);
     return $db->row_hash($key);
 }
@@ -411,71 +409,6 @@ TAGBUILD: {
 	}
 }
 
-# SQL
-sub sql_query {
-	my($type, $query, $opt, $list) = @_;
-
-	my ($db);
-
-	$opt->{table} = $Vend::Cfg->{ProductFiles}[0] unless defined $opt->{table};
-	$db = database_exists_ref($opt->{table})
-		or die "dbi_query: unknown base table $opt->{table}.\n";
-
-	$type = lc $type;
-	$type ||= 'list';
-
-	if ($list and $type ne 'list') {
-		$query = '' if ! defined $query;
-		$query .= $list;
-	}
-
-	my $perlquery;
-
-	my @arg;
-	while ($query =~ s:\[arg\](.*?)\[/arg\]::o) {
-		push(@arg, $1);
-	}
-
-	while ($query =~ s:\[control\s+([\w][-\w]*)\]([\000-\377]*?)\[/control\]::) {
-		my ($key, $val) = ($1, $2);
-		if($key =~ /PERL/i) {
-			$perlquery = 1;
-			$opt->{textref} = 0;
-		}
-		elsif ($key =~ /BOTH/i){
-			$perlquery = 1;
-			$opt->{textref} = 1;
-		}
-		else {
-			$opt->{"\L$key"} = $val;
-		}
-	}
-
-	if($type eq 'list') {
-		$opt->{list} = 1;
-		push(@arg, $1) while $query =~ s:\[arg\](.*?)\[/arg\]::o;
-		$list =~ s:\[query\]([\000-\377]+)\[/query\]::i and $query = $1;
-	}
-	elsif ($type eq 'hash') {
-		$opt->{textref} = 1;
-		$opt->{hashref} = 1;
-	}
-	elsif ($type eq 'array') {
-		$opt->{textref} = 1;
-	}
-	elsif ($type eq 'param') {
-		warn "sql query type=param deprecated";
-		$opt->{list} = 1;
-		$list = '"[sql-code]" ';
-	}
-	elsif ($type eq 'html') {
-		$opt->{html} = 1;
-	}
-	$opt->{query} = $query if $query;
-	return $db->query($opt, $list, @arg);
-}
-# END SQL
-
 sub column_index {
     my ($field_name) = @_;
 	$Products = $Products->ref();
@@ -1095,7 +1028,7 @@ sub index_database {
 	my $db;
 	$db = database_exists_ref($dbname)
 		or do {
-			logError("Vend::Data export: non-existent database %s", $db);
+			logError("Vend::Data export: non-existent database %s", $dbname);
 			return undef;
 		};
 
@@ -1202,14 +1135,13 @@ EOF
 
 sub export_database {
 	my($db, $file, $type, $opt) = @_;
-	my(@data);
-	my ($field, $delete);
 	return undef unless defined $db;
 
+	my (@data, $field, $delete);
+
 	$field  = $opt->{field}         if $opt->{field};
 	$delete = $opt->{delete}		if $opt->{delete};
 
-
 	$db = database_exists_ref($db)
 		or do {
 			logError("Vend::Data export: non-existent database %s" , $db);
@@ -1218,6 +1150,18 @@ sub export_database {
 
 	$db = $db->ref();
 
+	if ($Vend::Cfg->{NoExportExternal}) {
+		# Skip export only for "external" tables (currently SQL and LDAP),
+		# just like NoImportExternal does
+		my $class = $db->config('Class');
+		my $class_config = $db_config{$class || $Global::Default_database};
+		return 1 if $class_config->{RestrictedImport};
+	}
+
+	my $table_name = $db->config('name');
+
+	return 1 if $Vend::Cfg->{NoExport}->{$table_name};
+
 	my $qual;
 	if($qual = $opt->{where}) {
 		if(ref $qual) {
@@ -1233,7 +1177,6 @@ sub export_database {
 		}
 	}
 
-	my $table_name = $db->config('name');
 	my $notes;
 	if("\U$type" eq 'NOTES') {
 		$type = 2;
@@ -1470,7 +1413,7 @@ sub chain_cost {
 	else {
 		@p = Text::ParseWords::shellwords($raw);
 	}
-	if(scalar @p > ($Vend::Cfg->{Limit}{chained_cost_levels} || 64)) {
+	if(scalar @p > ($::Limit->{chained_cost_levels} || 64)) {
 		logError('Too many chained cost levels for item ' .  uneval($item) );
 		return undef;
 	}
@@ -1482,7 +1425,7 @@ sub chain_cost {
 CHAIN:
 	foreach $price (@p) {
 		next if ! length($price);
-		if($its++ > ($Vend::Cfg->{Limit}{chained_cost_levels} || 64)) {
+		if($its++ > ($::Limit->{chained_cost_levels} || 64)) {
 			logError('Too many chained cost levels for item ' .  uneval($item) );
 			last CHAIN;
 		}
@@ -1505,6 +1448,10 @@ CHAIN:
 			my $mod = $1;
 			if($mod =~ s/^\$(\d|$)/$1/) {
 				$price = $item->{mv_price} || $mod;
+				if($price =~ /^\s*free\s*$/i) {
+					$final = 0;
+					last CHAIN;
+				}
 				redo CHAIN;
 			}
 			elsif($mod =~ /^(\w*):([^:]*)(?::(\S*))?$/) {
@@ -1626,7 +1573,7 @@ CHAIN:
 			elsif ($mod =~ s/^=([\d.]*)=([^=]+)//) {
 				$final += $1 if $1;
 				my ($attribute, $table, $field, $key) = split /:/, $2;
-				if($item->{$attribute}) {
+				if($attribute) {
 					$key = $field ? $item->{$attribute} : $item->{code}
 						unless $key;
 					$price = database_field( ( $table ||
@@ -2116,7 +2063,7 @@ sub update_data {
 										}
 										)
 					or do {
-						 logError("%s is not a file.", $dref->[0]);
+						 logError("%s is not a file (does form allow file upload?).", $dref->[0]);
 						 next;
 					};
 				Vend::Interpolate::tag_value_extended(
@@ -2307,6 +2254,11 @@ sub update_data {
 #::logDebug("storing d=$d $field blob_only=$CGI::values{mv_blob_only}");
 					($d, $f) = set_db($base_db, $field);
 #::logDebug("storing table=$table d=$d f=$f key=$key");
+
+					if(! $value and ! length($value)) {
+						$value = $CGI::values{"mv_data_undef:$field"} ? undef : '';
+					}
+
 					if(! defined $qd->{$d}) {
 						$qd->{$d} = $d;
 						$qf->{$d} = [$f];
