@@ -1,8 +1,8 @@
 # Vend::Dispatch - Handle Interchange page requests
 #
-# $Id: Dispatch.pm,v 1.60.2.2 2006/12/07 23:15:58 jon Exp $
+# $Id: Dispatch.pm,v 1.86 2007/08/12 07:00:43 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 2002 Mike Heins <mike@perusion.net>
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,13 +20,13 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Dispatch;
 
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 1.60.2.2 $, 10);
+$VERSION = substr(q$Revision: 1.86 $, 10);
 
 use POSIX qw(strftime);
 use Vend::Util;
@@ -705,26 +705,21 @@ EOF
 }
 
 sub run_in_catalog {
-	my ($cat, $job, $itl) = @_;
+	my ($cat, $job, $itl, $parms) = @_;
 	my ($g,$c);
 
 #::logGlobal("running job in cat=$cat");
+	$parms ||= {};
+	
 	$g = $Global::Catalog{$cat};
 	unless (defined $g) {
 		logGlobal( "Can't find catalog '%s'" , $cat );
 		return undef;
 	}
 
-	#$Vend::Log_suppress = 1;
-
-	unless ($Vend::Quiet) {
-		logGlobal("Run catalog '%s' jobs group=%s", $cat, $job || 'INTERNAL');
-	}
-	#undef $Vend::Log_suppress;
-
 	open_cat($cat);
 
-	logError("Run jobs group=%s", $job || 'INTERNAL');
+	logError("Run jobs group=%s pid=$$", $job || 'INTERNAL');
 
 	my $jobscfg = $Vend::Cfg->{Jobs};
 
@@ -734,7 +729,7 @@ sub run_in_catalog {
 		my ($d, $global_dir, $tmp);
 		my @jobdirs = ([$jobscfg->{base_directory} || 'etc/jobs', 0]);
 
-		if ($jobscfg->{use_global}) {
+		if (is_yes($jobscfg->{use_global})) {
 			push (@jobdirs, ["$Global::ConfDir/jobs", 1]);
 		}
 
@@ -756,6 +751,7 @@ sub run_in_catalog {
 			my @f = glob("$dir/*");
 			@f = grep ! -d $_, @f;
 			@f = grep $_ !~ /$Vend::Cfg->{HTMLsuffix}$/, @f;
+			@f = grep $_ =~ /$jobscfg->{suffix}$/, @f;
 			for(@f) {
 #::logGlobal("found jobs piece file=$_");
 				push @itl, [$_, readfile($_)];
@@ -771,7 +767,7 @@ sub run_in_catalog {
 		push @itl, ["Passed ITL", $itl];
 	}
 
-	my @out;
+	my (@out, $errors);
 
 	# remove bogus session created by logError
 	undef $Vend::Session;
@@ -789,44 +785,56 @@ sub run_in_catalog {
 				::logError ("Invalid jobs tracking database $jobscfg->{trackdb}");
 			}
 		}
+
+		eval {
+			# Run once at beginning
+			run_macro($jobscfg->{initialize});
+
+			# initialize or autoload can create session
+			# but must handle all aspects
+			unless ($Vend::Session) {
+				$CGI::values{mv_tmp_session} = 1;
+				init_session();
+			}
+
+			$CGI::remote_addr ||= 'none';
+			$CGI::useragent   ||= 'commandline';
+
+			for(@itl) {
+				# Run once at beginning of each job
+				run_macro($jobscfg->{autoload});
+
+				push @out, interpolate_html($_->[1]);
+
+				# Run once at end of each job
+				run_macro($jobscfg->{autoend});
+			}
+		};
+
+		if ($@) {
+			# job terminated due to an error
+			$errors = 1;
+
+			logError ("Job group=%s pid=$$ terminated with an error: %s", $job || 'INTERNAL', $@);
+			
+			# remove flag for this job
+			Vend::Server::flag_job($$, $cat, 'furl');
+		}
 		
-		# Run once at beginning
-		run_macro($jobscfg->{initialize});
-
-		# initialize or autoload can create session
-		# but must handle all aspects
-		unless ($Vend::Session) {
-			$CGI::values{mv_tmp_session} = 1;
-			init_session();
-		}
-
-		$CGI::remote_addr ||= 'none';
-		$CGI::useragent   ||= 'commandline';
-
-		for(@itl) {
-			# Run once at beginning of each job
-			run_macro($jobscfg->{autoload});
-
-			push @out, interpolate_html($_->[1]);
-
-			# Run once at end of each job
-			run_macro($jobscfg->{autoend});
-		}
-
 		if ($trackid) {
 			$trackdb->set_field($trackid, 'end_run',
 								Vend::Interpolate::mvtime(undef, {}, '%Y-%m-%d %H:%M'));
 		}
 	}
 	else {
-		logGlobal("Empty job=%s", $job);
+		logError("Empty job=%s", $job);
 	}
 	my $out = join "", @out;
 	my $filter = $jobscfg->{filter} || 'strip';
 	$out = Vend::Interpolate::filter_value($filter, $out);
-	$out .= full_dump() if $jobscfg->{add_session};
+	$out .= full_dump() if is_yes($jobscfg->{add_session});
 
-	logError("Finished jobs group=%s", $job || 'INTERNAL');
+	logError("Finished jobs group=%s pid=$$", $job || 'INTERNAL');
 	
 	close_cat();
 
@@ -834,7 +842,7 @@ sub run_in_catalog {
 	# no output (in spirit of the cron daemon)
 	return unless $out;
 	
-	if(my $addr = $Vend::JobsEmail || $jobscfg->{email}) {
+	if(my $addr = $parms->{email} || $jobscfg->{email}) {
 		my $subject = $jobscfg->{subject} || 'Interchange results for job: %s';
 		$subject = errmsg($subject, $job);
 		my $from = $jobscfg->{from} || $Vend::Cfg->{MailOrderTo};
@@ -1122,6 +1130,8 @@ EOF
 		$Vend::Xquote = '';
 	}
 
+	$::Limit = $Vend::Cfg->{Limit} || {};
+
 	chdir $Vend::Cfg->{VendRoot} 
 		or die "Couldn't change to $Vend::Cfg->{VendRoot}: $!\n";
 	POSIX::setlocale(POSIX::LC_ALL, $Vend::Cfg->{ExecutionLocale});
@@ -1130,6 +1140,20 @@ EOF
 
 #show_times("end cgi and config mapping") if $Global::ShowTimes;
 	open_database();
+
+	if (my $subname = $Vend::Cfg->{SpecialSub}{catalog_init}) {
+#::logDebug(errmsg("running subroutine '%s' for %s", $subname, 'catalog_init'));
+		my $sub = $Vend::Cfg->{Sub}{$subname} || $Global::GlobalSub->{$subname};
+		my $status;
+		eval {
+			$status = $sub->();
+		};
+
+		if($@) {
+			::logError("Error running %s subroutine %s: %s", 'catalog_init', $subname, $@);
+		}
+	}
+
 #show_times("end open_database") if $Global::ShowTimes;
 	return 1;
 }
@@ -1191,10 +1215,12 @@ sub dispatch {
 	## If returns false then was a 404 no catalog or a delivered image
 	open_cat() or return 1;
 
+	Vend::Server::set_process_name("$Vend::Cat $CGI::host");
+
 	$CGI::user = Vend::Util::check_authorization($CGI::authorization)
 		if defined $CGI::authorization;
 
-    my($sessionid, $seed);
+	my($sessionid, $seed);
 
 	$sessionid = $CGI::values{mv_session_id} || undef
 		and $sessionid =~ s/\0.*//s;
@@ -1236,6 +1262,8 @@ sub dispatch {
 		$Vend::CookieID = $Vend::Cookie = 1;
     }
 
+	Vend::Server::set_process_name("$Vend::Cat $CGI::host $sessionid");
+
 	$::Instance->{CookieName} = 'MV_SESSION_ID' if ! $::Instance->{CookieName};
 
 	$CGI::host = 'nobody' if $Vend::Cfg->{WideOpen};
@@ -1309,9 +1337,13 @@ RESOLVEID: {
 				# do nothing, no host checking
 			}
 			elsif(! $compare_host) {
-				new_session($seed) unless $CGI::secure;
-				init_session();
-				$Vend::Session->{shost} = $CGI::remote_addr;
+				if ($CGI::secure) {
+				    $Vend::Session->{shost} = $CGI::remote_addr;
+				}
+				else {
+				    new_session($seed);
+				    init_session();
+				}
 			}
 			elsif ($compare_host ne $CGI::remote_addr) {
 				new_session($seed);
@@ -1329,7 +1361,7 @@ RESOLVEID: {
 			last RESOLVEID;
 		}
 		elsif($Vend::Cfg->{RobotLimit}) {
-			if ($now - $Vend::Session->{'time'} > 30) {
+			if ($now - $Vend::Session->{'time'} > ($::Limit->{lockout_reset_seconds} || 30) ) {
 				$Vend::Session->{accesses} = 0;
 			}
 			else {
@@ -1339,49 +1371,53 @@ RESOLVEID: {
 					and ! $Vend::admin
 					)
 				{
-					my $msg = errmsg(
-			"WARNING: POSSIBLE BAD ROBOT. %s accesses with no 30 second pause.",
-			$Vend::Session->{accesses},
-					);
-					::logError($msg);
 					do_lockout();
 				}
 			}
 		}
-    }
-	else {
-		if($Vend::Cfg->{RobotLimit}) {
-			if (Vend::Session::count_ip() > $Vend::Cfg->{RobotLimit}) {
-				my $msg;
-				# Here they can get it back if they pass expiration time
-				my $wait = $Vend::Cfg->{Limit}{robot_expire} || 1;
-				$wait *= 24;
-				$msg = errmsg(<<EOF, $wait); 
+	} else {
+	    if (Vend::Session::count_ip() && !do_lockout()) {
+		my $msg;
+		# Here they can get it back if they pass expiration time
+		my $wait = $::Limit->{robot_expire} || 1;
+		$wait *= 24;
+		$msg = errmsg(<<EOF, $wait); 
 Too many new ID assignments for this IP address. Please wait at least %d hours
 before trying again. Only waiting that period will allow access. Terminating.
 EOF
-				$msg = get_locale_message(403, $msg);
-				do_lockout();
+		$msg = get_locale_message(403, $msg);
 
-				::logError('Too many IDs, %d hour wait enforced.', $wait);
+		::logError('Too many IDs, %d hour wait enforced.', $wait);
 
-				$Vend::StatusLine = <<EOF;
+		$Vend::StatusLine = <<EOF;
 Status: 403 Forbidden
 Content-Type: text/plain
 EOF
-					response($msg);
-					close_cat();
-					return;
-			}
-		}
-		new_session();
-    }
+		response($msg);
+		close_cat();
+		return;
+	    }
+	    new_session();
+	}
+
 }
 
 #::logDebug("session name='$Vend::SessionName'\n");
 
 	$Vend::Calc_initialized = 0;
 	$CGI::values{mv_session_id} = $Vend::Session->{id} = $Vend::SessionID;
+
+	if($Vend::admin and my $subname = $Vend::Cfg->{SpecialSub}{admin_init}) {
+		my $sub = $Vend::Cfg->{Sub}{$subname} || $Global::GlobalSub->{$subname};
+		eval {
+			$sub->();
+		};
+
+		if($@) {
+			::logError("Error running %s subroutine %s: %s", 'admin_init', $subname, $@);
+		}
+	}
+
 	if(my $vspace = $CGI::values{mv_values_space}) {
 		$::Values = $Vend::Session->{values_repository}{$vspace} ||= {};
 		$Vend::ValuesSpace = $vspace;
@@ -1389,18 +1425,44 @@ EOF
 
 	$Vend::Session->{'arg'} = $Vend::Argument = ($CGI::values{mv_arg} || undef);
 
+	my $new_source;
 	if ($CGI::values{mv_pc} and $CGI::values{mv_pc} =~ /\D/) {
-		$Vend::Session->{source} =	$CGI::values{mv_pc} eq 'RESET'
-										? ''
-										: $CGI::values{mv_pc};
+		$new_source = $Vend::Session->{source} = $CGI::values{mv_pc} eq 'RESET'
+											   ? ''
+											   : $CGI::values{mv_pc};
 	}
 	elsif($CGI::values{mv_source}) {
-		$Vend::Session->{source} =	$CGI::values{mv_source};
+		$new_source = $Vend::Session->{source} = $CGI::values{mv_source};
+	}
+	if ($new_source and $CGI::request_method eq 'GET' and $Vend::Cfg->{BounceReferrals}) {
+		my $path = $CGI::path_info;
+		$path =~ s:^/::;
+		my $form =
+			join '',
+			map { "$_=$CGI::values{$_}\n" }
+			sort keys %$CGI::values;
+		my $url = vendUrl($path, undef, undef, { form => $form, match_security => 1 });
+		my $msg = get_locale_message(
+			301,
+			"Redirected to %s.",
+			$url,
+		);
+		$Vend::StatusLine = <<EOF;
+Status: 301 Moved
+Location: $url
+Content-Type: text/plain
+
+Redirecting to $url
+EOF
+		response($msg);
+#::logDebug("bouncing to $url");
+		close_cat();
+		return;
 	}
 
 	$Vend::Session->{'user'} = $CGI::user;
 
-	undef $Vend::Cookie if 	$Vend::Session->{logged_in};
+	Vend::Server::set_process_name("$Vend::Cat $CGI::host $sessionid " . $Vend::Session->{username} || '-');
 
 	$CGI::pragma = 'no-cache'
 		if delete $::Scratch->{mv_no_cache};
@@ -1416,11 +1478,13 @@ EOF
 			if delete $Vend::Session->{one_time_path_alias}{$Vend::FinalPath};
 	}
 
-    url_history($Vend::FinalPath) if $Vend::Cfg->{History};
+	url_history($Vend::FinalPath) if $Vend::Cfg->{History};
+
+	Vend::Server::set_process_name("$Vend::Cat $CGI::host $sessionid " . ($Vend::Session->{username} || '-') . " $Vend::FinalPath");
 
 # TRACK
-    $Vend::Track = new Vend::Track
-		unless $Vend::admin and ! $::Variable->{MV_TRACK_ADMIN};
+	$Vend::Track = new Vend::Track
+		if $Vend::Cfg->{UserTrack} and not ($Vend::admin and ! $::Variable->{MV_TRACK_ADMIN});
 # END TRACK
 
 	if($Vend::Cfg->{DisplayErrors} and $Global::DisplayErrors) {
@@ -1527,6 +1591,7 @@ EOF
 	$Vend::FinalPath =~ s:^/+::;
 	$Vend::FinalPath =~ s/(\.html?)$//
 		and $Vend::Extension = $1;
+	$Vend::Session->{extension} = $1 || '';
 
 	my $record;
 	my $adb;
@@ -1573,7 +1638,6 @@ EOF
 
 	}
 
-	$Vend::Session->{extension} = $1 || '';
 #::logDebug("path=$Vend::FinalPath mv_action=$CGI::values{mv_action}");
 
   DOACTION: {
@@ -1591,7 +1655,7 @@ EOF
 	}
 
 #::logGlobal("action=$Vend::Action path=$Vend::FinalPath");
-	my ($sub, $status, $action);
+	my ($sub, $status);
 	if(defined $Vend::Cfg->{ActionMap}{$Vend::Action}) {
 		$sub = $Vend::Cfg->{ActionMap}{$Vend::Action};
 		$CGI::values{mv_nextpage} = $Vend::FinalPath
@@ -1637,9 +1701,8 @@ EOF
 	do_page() if $status;
 #show_times("end page display") if $Global::ShowTimes;
 
-	if(my $macro = $Vend::Cfg->{AutoEnd}) {
-		run_macro($macro);
-#show_times("end AutoEnd macro") if $Global::ShowTimes;
+	for my $routine (@{$Vend::Cfg->{CleanupRoutines}}) {
+		$routine->();
 	}
   }
 
@@ -1649,6 +1712,8 @@ EOF
 
 	close_cat();
 
+	Vend::Server::set_process_name('done');
+
 	undef $H;
 
 #show_times("end dispatch cleanup") if $Global::ShowTimes;
