@@ -1,8 +1,8 @@
 # Vend::Error - Handle Interchange error pages and messages
 # 
-# $Id: Error.pm,v 2.10 2005/11/08 18:14:45 jon Exp $
+# $Id: Error.pm,v 2.15 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Error;
 
@@ -38,7 +38,7 @@ use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = substr(q$Revision: 2.10 $, 10);
+$VERSION = substr(q$Revision: 2.15 $, 10);
 
 sub get_locale_message {
 	my ($code, $message, @arg) = @_;
@@ -156,6 +156,35 @@ sub full_dump {
 sub do_lockout {
 	my ($cmd);
 	my $msg = '';
+
+	# If the lockout SpecialSub exists, it is run. If it returns 
+	# true, we return now. If it returns false, we run the lockout
+	# as normal.
+	if (my $subname = $Vend::Cfg->{SpecialSub}{lockout}) {
+		::logDebug(errmsg("running subroutine '%s' for lockout", $subname));
+		my $sub = $Vend::Cfg->{Sub}{$subname} || $Global::GlobalSub->{$subname};
+		my $status;
+		eval {
+			$status = $sub->();
+		};
+
+		if($@) {
+			::logError("Error running lockout subroutine %s: %s", $subname, $@);
+		}
+
+		return $status if $status;
+	}
+
+	# Now we log the error after custom lockout routine gets chance
+	# to bypass 
+	my $pause = $::Limit->{lockout_reset_seconds} || 30;
+	my $msg = errmsg(
+		"WARNING: POSSIBLE BAD ROBOT. %s accesses with no %d second pause.",
+		$Vend::Session->{accesses},
+		$pause,
+	);
+	::logError($msg);
+
 	if($cmd = $Global::LockoutCommand) {
 		my $host = $CGI::remote_addr;
 		$cmd =~ s/%s/$host/ or $cmd .= " $host";
@@ -166,7 +195,9 @@ sub do_lockout {
 		logGlobal({level => 'notice'}, $msg);
 	}
 	$Vend::Cfg->{VendURL} = $Vend::Cfg->{SecureURL} = 'http://127.0.0.1';
+	$Vend::LockedOut = 1;
 	logError($msg) if $msg;
+	return;
 }
 
 1;
