@@ -1,8 +1,8 @@
 # Vend::File - Interchange file functions
 #
-# $Id: File.pm,v 2.21 2005/11/08 18:14:45 jon Exp $
+# $Id: File.pm,v 2.25 2007/08/09 13:40:53 pajamian Exp $
 # 
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::File;
 require Exporter;
@@ -55,7 +55,7 @@ use File::Path;
 use File::Copy;
 use subs qw(logError logGlobal);
 use vars qw($VERSION @EXPORT @EXPORT_OK $errstr);
-$VERSION = substr(q$Revision: 2.21 $, 10);
+$VERSION = substr(q$Revision: 2.25 $, 10);
 
 sub writefile {
     my($file, $data, $opt) = @_;
@@ -233,7 +233,7 @@ sub flock_lock {
     my $flag = $excl ? $flock_LOCK_EX : $flock_LOCK_SH;
 
     if ($wait) {
-	my $trylimit = $Vend::Cfg->{Limit}{file_lock_retries} || 5;
+	my $trylimit = $::Limit->{file_lock_retries} || 5;
 	my $failedcount = 0;
         while (
                 ! flock($fh, $flag)
@@ -398,6 +398,17 @@ sub absolute_or_relative {
     $file =~ $abspat or $file =~ $relpat;
 }
 
+sub make_absolute_file {
+	my ($path, $global) = @_;
+	# empty string stays empty
+	return unless length($path);
+	# is file already an absolute path?
+	return $path if $path =~ $abspat;
+	# use global or catalog root?
+	my $prefix = ($global ? $Global::VendRoot : $Vend::Cfg->{VendRoot});
+	return catfile($prefix, $path);
+}
+
 sub win_catfile {
     my $file = pop @_;
     return $file unless @_;
