@@ -1,8 +1,8 @@
 # Vend::Form - Generate Form widgets
 # 
-# $Id: Form.pm,v 2.64.2.1 2005/11/30 15:01:56 racke Exp $
+# $Id: Form.pm,v 2.71 2007/08/10 12:05:34 racke Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Form;
 
@@ -39,7 +39,7 @@ use vars qw/@ISA @EXPORT @EXPORT_OK $VERSION %Template %ExtraMeta/;
 require Exporter;
 @ISA = qw(Exporter);
 
-$VERSION = substr(q$Revision: 2.64.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.71 $, 10);
 
 @EXPORT = qw (
 	display
@@ -506,7 +506,7 @@ sub date_widget {
 		if($ey < 100) {
 			$ey += $cy;
 		}
-		@Years = ($by .. $ey);
+		@Years = $by <= $ey ? ($by .. $ey) : reverse ($ey .. $by);
 	}
 	if ($opt->{blank}) {
 		$out .= qq{<option value="0000"$opt_extra>----</option>};
@@ -521,7 +521,10 @@ sub date_widget {
 	$out .= qq{</select>};
 	return $out unless $opt->{time};
 
-	$val =~ s/^\d{8}//;
+	$val =~ s/^(\d{8})//;
+	# If the date is blank (0000-00-00), treat time of 00:00 as blank,
+	# not midnight, in the option selection below
+	my $blank_time = ($opt->{blank} and $1 !~ /[1-9]/);
 	$val =~ s/\D+//g;
 	$val = round_to_fifteen($val);
 	$out .= qq{<input type="hidden" name="$name" value=":">};
@@ -564,7 +567,7 @@ sub date_widget {
 	}
 	$opt->{start_hour}	||= 0;
 	$opt->{end_hour}	||= 23;
-	
+
 	for my $hr ( $opt->{start_hour} .. $opt->{end_hour} ) {
 		next if defined $opt->{start_hour} and $hr < $opt->{start_hour};
 		next if defined $opt->{end_hour} and $hr > $opt->{end_hour};
@@ -595,7 +598,7 @@ sub date_widget {
 			($out .= $o, next) unless ! $sel and $val;
 #::logDebug("prospect=$time actual=$val");
 			$o =~ s/>/ SELECTED>/ && $sel++
-				if $val eq $time;
+				if ! $blank_time and $val eq $time;
 			$out .= $o;
 		}
 	}
@@ -1032,7 +1035,6 @@ sub options_to_array {
 		for(@$passed) {
 			push @out, [split /\s*=\s*/, HTML::Entities::decode($_), 2];
 		}
-		return \@out;
 	}
 	elsif (ref $passed eq 'HASH') {
 		my @keys;
@@ -1057,11 +1059,18 @@ sub options_to_array {
 		for(@keys) {
 			push @out, [$_, $passed->{$_}];
 		}
-		return \@out;
 	}
 	else {
 		die "bad data type to options_to_array";
 	}
+
+	if ($opt->{applylocale}) {
+		for (@out) {
+			$_->[1] = errmsg($_->[1]);
+		}
+	}
+
+	return \@out;
 }
 
 sub display {
@@ -1320,10 +1329,10 @@ if($opt->{debug}) {
 		if($opt->{extra}) {
 			$opt->{extra} =~ s{(^|\s+)class=(["'])?[^\s'"]+\2}{$1};
 			$opt->{extra} =~ s/\s+$//;
-			$opt->{extra} .=	qq{class="$opt->{class}"} ;
+			$opt->{extra} .= qq{ class="$opt->{class}"};
 		}
 		else {
-			$opt->{extra} =	qq{class="$opt->{class}"} ;
+			$opt->{extra} = qq{class="$opt->{class}"};
 		}
 	}
 
