@@ -1,6 +1,6 @@
 # Vend::Menu - Interchange menu processing routines
 #
-# $Id: Menu.pm,v 2.46.2.1 2006/03/14 19:17:37 mheins Exp $
+# $Id: Menu.pm,v 2.50 2007/03/30 11:39:45 pajamian Exp $
 #
 # Copyright (C) 2002 Mike Heins, <mike@perusion.net>
 #
@@ -16,12 +16,12 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Menu;
 
-$VERSION = substr(q$Revision: 2.46.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.50 $, 10);
 
 use Vend::Util;
 use strict;
@@ -479,10 +479,7 @@ sub old_simple {
 			push @$fn, 'mv_last_row';
 			$list->[-1][$#$fn] = 1;
 		}
-		$main = '';
-		for(@$list) {
-			$main .= menu_link($template, $_, $opt);
-		}
+		$main = join($opt->{joiner}, map {menu_link($template, $_, $opt)} @$list);
 	}
 
 	# Prevent possibility of memory leak
@@ -661,7 +658,9 @@ EOF
 		out += '>';
 		var tstyle = ${vpf}link_style;
 		var tclass = ${vpf}link_class;
-		var ttarget = ${vpf}link_target;
+		var ttarget = l[${vpf}TARGET];
+		if(! ttarget)
+			ttarget = ${vpf}link_target;
 		var tprepend = ${vpf}link_prepend;
 		if(l[${vpf}PAGE]) {
 			out = out + '<a href="' + tprepend + l[ ${vpf}PAGE ] + '"';
@@ -1233,7 +1232,9 @@ function ${vpf}tree_link (idx) {
 
 	var tstyle = ${vpf}link_style;
 	var tclass = ${vpf}link_class;
-	var ttarget = ${vpf}link_target;
+	var ttarget = l[${vpf}TARGET];
+	if(! ttarget)
+		ttarget = ${vpf}link_target;
 	var tprepend = ${vpf}link_prepend;
 	if(l[${vpf}MV_CHILDREN] > 0) {
 		if(l[${vpf}MV_LEVEL] && ${vpf}icon_by_type) {
@@ -1889,7 +1890,9 @@ sub tree_line {
 							description
 							img_up
 							img_dn
-							img_sel / ];
+							img_sel
+							target
+							/ ];
 		if($opt->{loopfields}) {
 			if(! ref($opt->{loopfields})) {
 				my $fstring = $opt->{loopfields};
