@@ -1,8 +1,8 @@
 # Vend::Options - Interchange item options base module
 #
-# $Id: Options.pm,v 2.5 2005/11/08 18:14:45 jon Exp $
+# $Id: Options.pm,v 2.8 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,13 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Options;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.5 $, 10);
+$VERSION = substr(q$Revision: 2.8 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -89,7 +89,7 @@ sub find_options_type {
 		if(! defined $item->{$attrib}) {
 			$tab = $item->{mv_ib} || product_code_exists_tag($sku)
 					or do {
-						logOnce("options: Unknown product %s.", $sku);
+						logOnce('error', "options: Unknown product %s.", $sku);
 						return;
 					};
 			$item->{$attrib} = tag_data($tab, $field, $sku);
