@@ -1,8 +1,8 @@
 # Vend::Options::Old48 - Interchange 4.8 compatible product options
 #
-# $Id: Old48.pm,v 1.10 2005/11/08 18:14:46 jon Exp $
+# $Id: Old48.pm,v 1.14 2007/08/09 13:40:55 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group <interchange@icdevgroup.org>
+# Copyright (C) 2002-2007 Interchange Development Group <interchange@icdevgroup.org>
 # Copyright (C) 2002-2003 Mike Heins <mikeh@perusion.net>
 
 # This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,13 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 #
 
 package Vend::Options::Old48;
 
-$VERSION = substr(q$Revision: 1.10 $, 10);
+$VERSION = substr(q$Revision: 1.14 $, 10);
 
 =head1 NAME
 
@@ -193,7 +193,7 @@ sub display_options_matrix {
 			$phony->{mv_sku} = $def[$i];
 
 			if ($opt->{label}) {
-				$ref->[4] = "<B>$ref->[4]</b>" if $opt->{bold};
+				$ref->[4] = "<b>$ref->[4]</b>" if $opt->{bold};
 				push @out, $ref->[4];
 			}
 			push @out, Vend::Interpolate::tag_accessories(
@@ -399,12 +399,12 @@ sub display_options_simple {
 		# skip unless o_value
 		next unless $ref->[3];
 		if ($opt->{label}) {
-			$ref->[4] = "<B>$ref->[4]</b>" if $opt->{bold};
+			$ref->[4] = "<b>$ref->[4]</b>" if $opt->{bold};
 			push @out, $ref->[4];
 		}
 		my $precursor = $opt->{report}
 					  ? "$ref->[2]$opt->{separator}"
-					  : qq{<input type=hidden name="mv_item_option" value="$ref->[2]">};
+					  : qq{<input type="hidden" name="mv_item_option" value="$ref->[2]">};
 		push @out, $precursor . Vend::Interpolate::tag_accessories(
 						$sku,
 						'',
@@ -437,4 +437,6 @@ sub display_options_simple {
 	return $out;
 }
 
+*display_options = \&display_options_simple;
+
 1;
