@@ -1,8 +1,8 @@
 # Vend::Order - Interchange order routing routines
 #
-# $Id: Order.pm,v 2.83.2.4 2007/01/31 00:13:36 pajamian Exp $
+# $Id: Order.pm,v 2.94 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -23,13 +23,13 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Order;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.83.2.4 $, 10);
+$VERSION = substr(q$Revision: 2.94 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -229,11 +229,6 @@ sub _and_check {
 	return chain_checks(0, @_);
 }
 
-sub _always_pass {
-	my($ref,$var,$val) = @_;
-	return (1, $var, '')
-}
-
 sub _or_check {
 	if(! length($_[1]) ) {
 		$And = 0;
@@ -465,6 +460,8 @@ sub guess_cc_type {
 	my ($ccnum) = @_;
 	$ccnum =~ s/\D+//g;
 
+	my $country = $::Values->{$::Variable->{MV_COUNTRY_FIELD} || 'country'} || '';
+
 	if(my $subname = $Vend::Cfg->{SpecialSub}{guess_cc_type}) {
 		my $sub = $Vend::Cfg->{Sub}{$subname} || $Global::GlobalSub->{$subname};
 		my $guess;
@@ -473,24 +470,50 @@ sub guess_cc_type {
 		}
 	}
 
-	# based on logic by Karl Moore from http://www.vb-world.net/tips/tip509.html
+	# based on logic from Business::CreditCard
 	if ($ccnum eq '')										{ '' }
-	elsif ($ccnum =~ /^4(?:\d{12}|\d{15})$/)				{ return 'visa' }
-	elsif ($ccnum =~ /^5[1-5]\d{14}$/)						{ return 'mc' }
-	elsif ($ccnum =~ /^6011\d{12}$/)						{ return 'discover' }
-	elsif ($ccnum =~ /^3[47]\d{10,13}$/)						{ return 'amex' }
-	elsif ($ccnum =~ /^3(?:6\d{12}|0[0-5]\d{11})$/)			{ return 'dinersclub' }
-	elsif ($ccnum =~ /^38\d{12}$/)							{ return 'carteblanche' }
-	elsif ($ccnum =~ /^2(?:014|149)\d{11}$/)				{ return 'enroute' }
-	elsif ($ccnum =~ /^(?:3\d{15}|2131\d{11}|1800\d{11})$/)	{ return 'jcb' }
-    elsif ($ccnum =~ /^49(03(0[2-9]|3[5-9])|11(0[1-2]|7[4-9]|8[1-2])|36[0-9]{2})\d{10}(\d{2,3})?$/
+	elsif (
+		$ccnum =~ /^4(?:\d{12}|\d{15})$/
+	)			
+	{
+		return 'visa';
+	}
+	elsif (
+		$ccnum =~ /^5[1-5]\d{14}$/
+			or
+		( $ccnum =~ /^36\d{12}/ && $country =~ /^(US|CA)$/oi )
+	)
+	{ return 'mc' }
+	elsif (
+		$ccnum =~ /^6011\d{12}$/
+			or
+		$ccnum =~ /^65\d{14}$/o
+	    or
+		( $ccnum =~ /^622\d{13}$/o && $country !~ /^CN$/i )
+
+		)
+	{ return 'discover' }
+	elsif ($ccnum =~ /^3[47]\d{10,13}$/)
+	{ return 'amex' }
+	elsif ($ccnum =~ /^3(?:6\d{12}|0[0-5]\d{11})$/)
+	{ return 'dinersclub' }
+	elsif ($ccnum =~ /^38\d{12}$/)
+	{ return 'carteblanche' }
+	elsif ($ccnum =~ /^2(?:014|149)\d{11}$/)
+	{ return 'enroute' }
+	elsif ($ccnum =~ /^(?:3\d{15}|2131\d{11}|1800\d{11})$/)
+	{ return 'jcb' }
+    elsif (
+		$ccnum =~ /^49(03(0[2-9]|3[5-9])|11(0[1-2]|7[4-9]|8[1-2])|36[0-9]{2})\d{10}(\d{2,3})?$/
       or $ccnum =~ /^564182\d{10}(\d{2,3})?$/
       or $ccnum =~ /^6(3(33[0-4][0-9])|759[0-9]{2})\d{10}(\d{2,3})?$/) 
-															{ return 'switch' }
-
-	elsif ($ccnum =~ /^56\d{14}$/)                          { return 'bankcard' }
+	{ return 'switch' }
+	elsif ($ccnum =~ /^56\d{14}$/)
+	{ return 'bankcard' }
     elsif ($ccnum =~ /^6(3(34[5-9][0-9])|767[0-9]{2})\d{10}(\d{2,3})?$/)
-		 { return 'solo' }
+	{ return 'solo' }
+    elsif ($ccnum =~ /^622\d{13}$/)
+	{ return 'chinaunionpay' }
 
 	else { return $::Variable->{MV_PAYMENT_OTHER_CARD} || 'other' }
 }
@@ -577,6 +600,13 @@ sub encrypt_standard_cc {
 		return @return;
 	}
 
+	unless ($num) {
+		my $msg = errmsg("Missing credit card number");
+		$Vend::Session->{errors}{mv_credit_card_valid} = $msg;
+		push @return, $msg;
+		return @return;
+	}
+	
 	$type = guess_cc_type($num) unless $type;
 
 	if ($type and $opt->{accepted} and $opt->{accepted} !~ /\b$type\b/i) {
@@ -1291,7 +1321,7 @@ sub _phone_us_with_area_strict {
 
 sub _email {
 	my($ref, $var, $val) = @_;
-	if($val and $val =~ /[\040-\176]+\@[-A-Za-z0-9.]+\.[A-Za-z]+/) {
+	if($val and $val =~ /[\040-\077\101-\176]+\@[-A-Za-z0-9.]+\.[A-Za-z]+/) {
 		return (1, $var, '');
 	}
 	else {
