@@ -1,8 +1,8 @@
 # Vend::Page - Handle Interchange page routing
 # 
-# $Id: Page.pm,v 2.21.2.1 2006/05/11 12:31:52 mheins Exp $
+# $Id: Page.pm,v 2.25 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Page;
 
@@ -46,7 +46,7 @@ use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = substr(q$Revision: 2.21.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.25 $, 10);
 
 my $wantref = 1;
 
@@ -73,7 +73,7 @@ sub display_special_page {
 	
 	$page = readfile($noname, $Global::NoAbsolute, 1) || readin($name);
 
-	die ::get_locale_message(412, "Missing special page: %s\n", $name)
+	die ::get_locale_message(412, qq{Missing special page "%s" for subject "%s"\n}, $name, $subject)
 		unless defined $page;
 	$page =~ s#\[subject\]#$subject#ig;
 	$Global::Variable->{MV_SUBJECT} = $subject;
