@@ -1,8 +1,8 @@
 # Vend::Parse - Parse Interchange tags
 # 
-# $Id: Parse.pm,v 2.37 2005/11/08 18:14:45 jon Exp $
+# $Id: Parse.pm,v 2.40 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Parse;
 require Vend::Parser;
@@ -36,7 +36,7 @@ require Exporter;
 
 @ISA = qw(Exporter Vend::Parser);
 
-$VERSION = substr(q$Revision: 2.37 $, 10);
+$VERSION = substr(q$Revision: 2.40 $, 10);
 
 @EXPORT = ();
 @EXPORT_OK = qw(find_matching_end);
@@ -816,7 +816,9 @@ EOF
 #::logDebug("FOUND end for $tag\nBuf " . length($$buf) . ":\n" . $$buf . "\nTmpbuf:\n$tmpbuf\n");
 		if ($attr->{interpolate} and !$empty_container) {
 			my $p = new Vend::Parse;
+			my $tagsave = $Vend::CurrentTag;
 			$p->parse($tmpbuf);
+			$Vend::CurrentTag = $tagsave;
 			$tmpbuf = $p->{ABORT} ? '' : ${$p->{OUT}};
 		}
 		if($attr->{reparse} ) {
