@@ -1,8 +1,8 @@
 # Vend::Payment - Interchange payment processing routines
 #
-# $Id: Payment.pm,v 2.16 2005/05/04 01:12:26 mheins Exp $
+# $Id: Payment.pm,v 2.19 2007/08/09 13:40:53 pajamian Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,13 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Payment;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.16 $, 10);
+$VERSION = substr(q$Revision: 2.19 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -148,6 +148,8 @@ sub map_actual {
 		mv_credit_card_exp_month
 		mv_credit_card_exp_year
 		mv_credit_card_number
+		mv_order_number
+		mv_transaction_id
 		name
 		origin_zip
 		phone_day
