@@ -1,8 +1,8 @@
 # Vend::Scan - Prepare searches for Interchange
 #
-# $Id: Scan.pm,v 2.31 2005/11/08 18:14:45 jon Exp $
+# $Id: Scan.pm,v 2.34 2007/08/09 13:40:54 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Scan;
 require Exporter;
@@ -30,7 +30,7 @@ require Exporter;
 			perform_search
 			);
 
-$VERSION = substr(q$Revision: 2.31 $, 10);
+$VERSION = substr(q$Revision: 2.34 $, 10);
 
 use strict;
 no warnings qw(uninitialized numeric);
@@ -540,9 +540,12 @@ sub perform_search {
 				$q = "$Global::Variable->{$options{mv_searchtype}}"->new(%options);
 			};
 			if ($@) {
+				::logError("Search initialization for search type %s failed: %s",
+						   $options{mv_searchtype}, $@);
+				
 				::display_special_page(
 					find_special_page('badsearch'),
-					errmsg("Bad search type %s: %s", $options{mv_searchtype}, $@ ),
+					errmsg('Search initialization failed')
 					);
 				return 0;
 			}
