@@ -1,8 +1,8 @@
 # Vend::Search - Base class for search engines
 #
-# $Id: Search.pm,v 2.30 2005/04/30 15:09:58 mheins Exp $
+# $Id: Search.pm,v 2.36 2007/08/09 13:40:54 pajamian Exp $
 #
-# Copyright (C) 2002-2004 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,18 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Search;
 
-$VERSION = substr(q$Revision: 2.30 $, 10);
+$VERSION = substr(q$Revision: 2.36 $, 10);
 
 use strict;
 no warnings qw(uninitialized numeric);
+
+use POSIX qw(LC_CTYPE);
+
 use vars qw($VERSION);
 
 sub new {
@@ -151,7 +154,7 @@ sub hash_fields {
 
 sub escape {
     my($s, @text) = @_;
-#::logDebug( "escaped text=" . ::uneval(\@text));
+#::logDebug( "unescaped text=" . ::uneval(\@text));
 	return @text if ! $s->{mv_all_chars}[0];
 	@text = map {quotemeta $_} @text;
 #::logDebug( "escaped text=" . ::uneval(\@text));
@@ -424,7 +427,7 @@ sub more_alpha {
 	$last = 0;
 	for ($i = 0; $i < @$out; $i++) {
 		$sortkey = $out->[$i]->[$sfpos];
-		@letters = split(//, $sortkey, $alphachars + 1);
+		@letters = split(m{}, $sortkey, $alphachars + 1);
 		pop(@letters);
 		$changed = 0;
 
@@ -649,13 +652,19 @@ sub map_ops {
 		$c->[$i] =~ tr/ \t//;
 		my $o = $c->[$i];
 		$c->[$i] = $s->{mv_numeric}[$i]
-				? [ @{$numopmap{$o}} ]
-				: [ @{$stropmap{$o}} ];
-		if(! $c->[$i]) {
+				? $numopmap{$o}
+				: $stropmap{$o};
+		if (ref($c->[$i]) eq 'ARRAY') {
+			$c->[$i] = [ @{$c->[$i]} ];
+		}
+		elsif (!$c->[$i]) {
 			my $r;
 			$c->[$i] = [$r, $o], next
 				if  $r = Vend::Util::codedef_routine('SearchOp',$o);
 		}
+		if (!$c->[$i]) {
+		    $s->search_error("Unknown mv_column_op (%s)",$o);
+		}
 	}
 	@{$s->{mv_column_op}};
 }
@@ -1016,6 +1025,9 @@ EOF
 	}
 #::logDebug("code is $code");
 	use locale;
+	if ($::Scratch->{mv_locale}) {
+	    POSIX::setlocale(LC_CTYPE, $::Scratch->{mv_locale});
+	}
 	$limit_sub = eval $code;
 	die "Bad code: $@" if $@;
 	return ($limit_sub, $f);
