@@ -1,8 +1,8 @@
 # Vend::Server - Listen for Interchange CGI requests as a background server
 #
-# $Id: Server.pm,v 2.66.2.1 2006/06/06 18:05:23 kwalsh Exp $
+# $Id: Server.pm,v 2.81 2007/08/20 21:42:21 racke Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,14 +20,15 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Server;
 
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 2.66.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.81 $, 10);
 
+use Cwd;
 use POSIX qw(setsid strftime);
 use Vend::Util;
 use Fcntl;
@@ -279,12 +280,14 @@ EOF
 			$CGI::values{mv_tmp_session} = 1;
 		}
 	}
-	elsif ($Global::NotRobotUA and $CGI::useragent =~ $Global::NotRobotUA) {
-		# do nothing
-	}
-	elsif ($Global::RobotUA and $CGI::useragent =~ $Global::RobotUA) {
+	unless ($CGI::values{mv_tmp_session}) { 
+		if ($Global::NotRobotUA and $CGI::useragent =~ $Global::NotRobotUA) {
+			# do nothing
+		}
+		elsif ($Global::RobotUA and $CGI::useragent =~ $Global::RobotUA) {
 #::logDebug("It is a robot by UA!");
-		$CGI::values{mv_tmp_session} = 1;
+			$CGI::values{mv_tmp_session} = 1;
+		}
 	}
 }
 
@@ -349,6 +352,7 @@ sub parse_post {
 
 #::logDebug("incoming --> $key");
 		$key = $::IV->{$key} if defined $::IV->{$key};
+		$key =~ tr/+/ /;
 		$key =~ s/%([0-9a-fA-F][0-9a-fA-F])/chr(hex $1)/ge;
 #::logDebug("mapping  --> $key");
 		$value =~ tr/+/ /;
@@ -868,29 +872,40 @@ sub read_cgi_data {
 sub connection {
     my (%env, $entity);
 
-  ### This resets all $Vend::variable settings so we start
-  ### completely initialized. It only affects the Vend package,
-  ### not any Vend::XXX packages.
-	reset_vars();
+    my $show_in_ps = shift;
 
-	if($Global::ShowTimes) {
-		@Vend::Times = times();
-		::logDebug ("begin connection. Summary time set to zero");
-	}
+    set_process_name('connection');
+
+    ### This resets all $Vend::variable settings so we start
+    ### completely initialized. It only affects the Vend package,
+    ### not any Vend::XXX packages.
+    reset_vars();
+
+    if($Global::ShowTimes) {
+	@Vend::Times = times();
+	::logDebug ("begin connection. Summary time set to zero");
+    }
     read_cgi_data(\@Global::argv, \%env, \$entity)
     	or return 0;
     show_times('end cgi read') if $Global::ShowTimes;
 
     my $http = new Vend::Server \*MESSAGE, \%env, \$entity;
 
-	# Can log all CGI inputs
-	log_http_data($http) if $Global::Logging;
+    # Can log all CGI inputs
+    log_http_data($http) if $Global::Logging;
 
-	show_times("begin dispatch") if $Global::ShowTimes;
+    set_process_name('dispatch');
+
+    show_times("begin dispatch") if $Global::ShowTimes;
     ::dispatch($http) if $http;
-	show_times("end connection") if $Global::ShowTimes;
-	close $http->{rfh} if $http->{rfh};
-	undef $Vend::Cfg;
+    show_times("end connection") if $Global::ShowTimes;
+    close $http->{rfh} if $http->{rfh};
+    undef $Vend::Cfg;
+
+    my $display = 'done';
+    $display .= "($show_in_ps)" if $show_in_ps;
+
+    set_process_name($display);
 }
 
 ## Signals
@@ -917,6 +932,9 @@ use vars qw(
 			$Num_servers
 			$Page_servers
 			%Page_pids
+			%Starting_pids
+			$Starting_pids
+			@Termed_pids
 			$SOAP_servers
 			%SOAP_pids
 			$Job_servers
@@ -929,6 +947,7 @@ use vars qw(
 BEGIN {
 	$s_vector = '';
 }
+$Starting_pids = 0;
 $Num_servers = 0;
 $SOAP_servers = 0;
 $Job_servers = 0;
@@ -939,6 +958,38 @@ $Job_servers = 0;
 my ($Routine_USR1, $Routine_USR2, $Routine_HUP, $Routine_TERM, $Routine_INT);
 my ($Sig_inc, $Sig_dec, $Counter);
 
+sub sig_int_or_term {
+	$Signal_Terminate = 1;
+
+	my $term_count = 0;
+	TERM: {
+		my %seen;
+		my @pids =
+			grep { !$seen{$_}++ }
+				(keys %Page_pids, keys %Starting_pids);
+
+		last TERM unless @pids;
+
+		kill TERM => $_ for @pids;
+		sleep 1;
+
+		redo TERM unless ++$term_count > 3;
+	}
+
+	KILL: {
+		my %seen;
+		my @pids =
+			grep { !$seen{$_}++ }
+				(keys %Page_pids, keys %Starting_pids);
+
+		last KILL unless @pids;
+
+		kill KILL => $_ for @pids;
+	}
+
+	return;
+}
+
 unless ($Global::Windows) {
 	push @trapped_signals, qw(HUP USR1 USR2);
 	$Routine_USR1 = sub { $SIG{USR1} = $Routine_USR1; $Num_servers++};
@@ -979,18 +1030,20 @@ sub clean_up_after_fork {
 }
 
 sub setup_signals {
-    @orig_signal{@trapped_signals} =
-        map(defined $_ ? $_ : 'DEFAULT', @SIG{@trapped_signals});
-    $Signal_Terminate = '';
-    $SIG{PIPE} = 'IGNORE';
+	@orig_signal{@trapped_signals} =
+		map(defined $_ ? $_ : 'DEFAULT', @SIG{@trapped_signals});
+	$Signal_Terminate = '';
+	$SIG{PIPE} = 'IGNORE';
+	$SIG{CHLD} = 'IGNORE'
+		if $Global::PreFork && $Global::PreForkSingleFork;
 
 	if ($Global::Windows) {
-		$SIG{INT}  = sub { $Signal_Terminate = 1; };
-		$SIG{TERM} = sub { $Signal_Terminate = 1; };
+		$SIG{INT}  = \&sig_int_or_term;
+		$SIG{TERM} = \&sig_int_or_term;
 	}
 	else  {
-		$SIG{INT}  = sub { $Signal_Terminate = 1; };
-		$SIG{TERM} = sub { $Signal_Terminate = 1; };
+		$SIG{INT}  = \&sig_int_or_term;
+		$SIG{TERM} = \&sig_int_or_term;
 		$SIG{HUP}  = sub { $Signal_Restart = 1; };
 		$SIG{USR1} = sub { $Num_servers++; };
 		$SIG{USR2} = sub { $Num_servers--; };
@@ -1041,45 +1094,84 @@ sub housekeeping {
 	rand();
 	$Last_housekeeping = $now;
 
-	my ($c, $num,$reconfig, $restart, $jobs, @files);
-	my @pids;
+	my ($c, $num,$reconfig, $restart, $jobs, @files, @pidcheck_pids);
 
 		if($Global::PreFork) {
-			my @bad_pids;
-			my (@pids) = sort keys %Page_pids;
-			my $count = scalar @pids;
-			while ($count > ($Global::StartServers + 1) ) {
-#::logDebug("too many pids");
-				my ($bad) = shift(@pids);
-#::logDebug("scheduling %s for death", $bad);
-				push @bad_pids, $bad;
-				$count--;
+			my @starting_pids = keys %Starting_pids;
+			my $starting_count = starting_pids('count');
+			my %bad_pids;
+			my @active_pids = keys %Page_pids;
+			my $active_count = scalar @active_pids;
+			my $check_time = time();
+			my $start_max_time = 30;
+
+			for my $pid (@starting_pids) {
+				my $time_taken = $check_time - $Starting_pids{$pid};
+				if ($time_taken > $start_max_time) {
+					::logDebug("pid $pid took $time_taken seconds to start ($start_max_time allowed); scheduling for death");
+					$bad_pids{$pid} = undef;
+					delete $Starting_pids{$pid};
+					--$starting_count;
+				}
 			}
 
-			foreach my $pid (@pids) {
+			while ($active_count > ($Global::StartServers + 1) ) {
+#::logDebug("too many pids ($active_count)");
+				my $bad = shift @active_pids;
+#::logDebug("scheduling %s for death", $bad);
+				$bad_pids{$bad} = undef;
+				--$active_count;
+			}
+
+			foreach my $pid (@active_pids) {
 				kill(0, $pid) and next;
 #::logDebug("Non-existent server at PID %s", $pid);
-				push @bad_pids, $pid;
 				delete $Page_pids{$pid};
+				--$active_count;
 			}
 
-			while($count < $Global::StartServers) {
-#::logDebug("Spawning page server to reach StartServers");
-				start_page(undef,$Global::PreFork,1);
-				$count++;
+			if ($Global::PIDcheck) {
+				for my $pid (keys %Page_pids) {
+					my $last_use = $check_time - $Page_pids{$pid};
+					next unless $last_use > $Global::PIDcheck;
+#::logDebug('pid %s last used %d seconds ago', $pid, $last_use);
+					$bad_pids{$pid} = undef;
+					delete $Page_pids{$pid};
+#::logDebug('scheduling %s for death', $pid);
+					--$active_count;
+				}
 			}
-			for my $pid (@bad_pids) {
-#::logDebug("Killing excess or unresponsive server at PID %s", $pid);
-				next unless delete $Page_pids{$pid};
-				if(kill 'TERM', $pid) {
-#::logDebug("Server at PID %s terminated OK", $pid);
-					# This is OK
-				}
-				elsif (kill 'KILL', $pid) {
-					::logGlobal("page server pid %s required KILL", $pid);
-				}
-				else {
-					::logGlobal("page server pid %s won't die!", $pid);
+
+			if ($active_count + $starting_count < $Global::StartServers) {
+				my $server_deficit =
+					$Global::StartServers
+					- $active_count
+					- $starting_count;
+				::logGlobal("Spawning %d page server%s to reach %s StartServers", $server_deficit, $server_deficit == 1 ? '' : 's', $Global::StartServers);
+				start_page(undef, $Global::PreFork, $server_deficit);
+			}
+
+			for my $pid (@Termed_pids) {
+				kill (KILL => $pid)
+					and ::logDebug("Sent $pid a KILL");
+			}
+			::logGlobal("page server pid %s won't die!", $_)
+					for grep { kill (0, $_) } @Termed_pids;
+			@Termed_pids = ();
+
+			if (%bad_pids) {
+#::logDebug("Killing excess, old, or unresponsive servers");
+				delete @Page_pids{ keys %bad_pids };
+
+				for my $pid
+					( grep
+						{ kill (0, $_) or delete $bad_pids{$_} }
+						keys %bad_pids
+					)
+				{
+					kill (TERM => $pid);
+					::logDebug("Sent $pid a TERM");
+					push (@Termed_pids, $pid);
 				}
 			}
 		}
@@ -1110,7 +1202,7 @@ sub housekeeping {
 
 		if($Global::PIDcheck) {
 			$Num_servers = 0;
-			@pids = grep /^pid\.\d+$/, @files;
+			@pidcheck_pids = grep /^pid\.\d+$/, @files;
 		}
 
 		my $respawn;
@@ -1232,7 +1324,7 @@ EOF
 			while(<Vend::Server::JOBS>) {
 				chomp;
 				my ($directive,$value) = split /\s+/, $_, 2;
-				my ($cat, $delay, @jobs) = grep /\S/, split /[\s,\0]+/, $value;
+				my ($cat, $delay, $jobname, @params) = grep /\S/, split /[\s,\0]+/, $value;
 				if ($delay && $delay < time()) {
 					# job expired
 #::logDebug ("Jobs @jobs expired ($delay vs $now)\n");
@@ -1243,7 +1335,12 @@ EOF
                         push(@queued_jobs, "$directive $value");
                 } else {
 #::logDebug ("Scheduled job @jobs for running");
-					push (@scheduled_jobs, [$cat, @jobs]);
+					my %p;
+					for (@params) {
+					    my ($name, $value) = split /\=/, $_, 2;
+						$p{$name} = $value;
+					}
+					push (@scheduled_jobs, [$cat, $jobname, \%p]);
 				}
                 if (@queued_jobs > 20) {
 					::logGlobal({ level => 'notice' }, "Excessive size of job queue, stopping");
@@ -1326,8 +1423,8 @@ EOF
 								$_,
 								$!,
 							);
-					start_page(undef,$Global::PreFork,1);
 				}
+				start_page(undef, $Global::PreFork, scalar @pids);
 			}
 			if($Global::SOAP) {
 				# We need to respawn all the SOAP servers to pick up the new config
@@ -1349,7 +1446,7 @@ EOF
 			}
 		}
 
-        for (@pids) {
+        for (@pidcheck_pids) {
             $Num_servers++;
             my $fn = "$Global::RunDir/$_";
             ($Num_servers--, next) if ! -f $fn;
@@ -1363,14 +1460,26 @@ EOF
 				$lifetime = $Global::PIDcheck;
 			}
             next if $runtime < $lifetime;
+			my $catname;
 			if ($isjob) {
-	            delete $Lifetime{$_};
-    	        $Job_servers--;
+				# determine catalog name from pid file
+				if (open (JOBPID, $fn)) {
+					$catname = <JOBPID>;
+					chomp($catname);
+					close (JOBPID);
+					delete $Lifetime{$_};
+					$Job_servers--;
+				}
 			}
-
+			
             if(kill 9, $_) {
                 unlink $fn and $Num_servers--;
-                ::logGlobal({ level => 'error' }, "hammered PID %s running %s seconds", $_, $runtime);
+				if ($catname) {
+					::logGlobal({ level => 'error' }, "hammered job PID %s for catalog $catname running %s seconds", $_, $runtime);
+					flag_job($_, $catname, 'furl');
+				} else {
+					::logGlobal({ level => 'error' }, "hammered PID %s running %s seconds", $_, $runtime);
+				}
             }
             elsif (! kill 0, $_) {
 				unlink $fn and $Num_servers--;
@@ -1450,7 +1559,7 @@ sub map_unix_socket {
 }
 
 sub map_inet_socket {
-	my ($vec, $vec_map, $fh_map, @ports) = @_;
+	my ($mode, $vec, $vec_map, $fh_map, @ports) = @_;
 
 	my $proto = getprotobyname('tcp');
 	my @made;
@@ -1501,7 +1610,8 @@ sub map_inet_socket {
 
 		if ($@) {
 		  ::logGlobal({ level => 'error' },
-					"INET mode server failed to start on port %s: %s",
+					"$mode mode server failed to start on IP address %s, port %s: %s",
+					$bind_ip,
 					$bind_port,
 					$@,
 				  );
@@ -1566,33 +1676,58 @@ sub unlink_sockets {
 sub start_page {
 
 	my ($do_message, $no_fork, $number) = @_;
-#::logDebug("starting soap");
+#::logDebug("entering start_page");
 
-	$number = $Global::StartServers if ! $number; 
-	if ($number > 50) {
+	my $current_servers =
+		starting_pids('count')
+		+ scalar (keys %Page_pids);
+
+	my $server_deficit = $Global::StartServers - $current_servers;
+
+	# Bail immediately if we already have a slate of
+	# StartServers servers either pending or serving
+	return 1 if $server_deficit < 1;
+
+	# Shave number down to server_deficit if it's greater
+	$number = $server_deficit if $server_deficit < $number;
+
+	if ($number > 150) {
 		  die ::errmsg(
 		   "Ridiculously large number of StartServers: %s",
 		   $number,
 		   );
 	}
-	for (1 .. $number) {
-		my $pid;
-		if(! defined ($pid = fork) ) {
-			my $msg = ::errmsg("Can't fork: %s", $!);
-			::logGlobal({ level => 'crit' },  $msg );
-			die ("$msg\n");
-		}
-		elsif (! $pid) {
-			unless( $pid = fork ) {
+	my $dbl_fork_pid;
+
+	if (
+			$Global::PreForkSingleFork
+			or ! ($dbl_fork_pid = fork)
+		)
+	{
+
+		for (1 .. $number) {
+			my $pid;
+			if(! defined ($pid = fork) ) {
+				my $msg = ::errmsg("Can't fork: %s", $!);
+				::logGlobal({ level => 'crit' },  $msg );
+				die ("$msg\n");
+			}
+			elsif (! $pid) {
 				$Global::Foreground = 1 if $no_fork;
 
-				if($do_message) {
+				local $SIG{CHLD} = 'DEFAULT'
+					if $Global::PreForkSingleFork;
+
+				local $SIG{INT} = $Routine_INT;
+				local $SIG{TERM} = $Routine_TERM;
+
+				if ($do_message and ! $Vend::Quiet) {
 					::logGlobal(
 						{ level => 'info'},
 						server_start_message(
 							"Interchange page server started (process id %s)",
-						 ),
-					 ) unless $Vend::Quiet;
+						),
+					);
 				}
 
 				send_ipc("register page $$");
@@ -1608,20 +1743,27 @@ sub start_page {
 				if ($@) {
 					my $msg = ::errmsg("Server spawn error: %s", $@);
 					::logGlobal({ level => 'error' }, $msg);
-					logError($msg)
+					::logError($msg)
 						if defined $Vend::Cfg->{ErrorFile};
 				}
 
 				clean_up_after_fork();
-				send_ipc("respawn page $$")		if $next;
+				send_ipc("respawn page $$") if $next;
 				
 				undef $::Instance;
 				exit(0);
 			}
-			exit(0);
+			starting_pids('add',$pid)
+				if $Global::PreForkSingleFork;
 		}
+		$Global::PreForkSingleFork or exit(0);
+	}
+
+	if ($dbl_fork_pid) {
+		starting_pids('add',undef,$number);
 		wait;
 	}
+
 	return 1;
 }
 
@@ -1688,6 +1830,32 @@ sub start_soap {
 	return 1;
 }
 
+sub starting_pids {
+	my ($action,$pid,$n) = @_;
+
+	$n ||= 1;
+
+	if ( $action eq 'count' ) {
+		return $Global::PreForkSingleFork
+			? scalar keys %Starting_pids
+			: $Starting_pids
+		;
+	}
+	elsif ( $action eq 'add' ) {
+		$Global::PreForkSingleFork
+			? ($Starting_pids{$pid} = time)
+			: ($Starting_pids += $n)
+		;
+	}
+	elsif ( $action eq 'del' ) {
+		$Global::PreForkSingleFork
+			? delete ($Starting_pids{$pid})
+			: ($Starting_pids -= $n)
+		;
+	}
+	return;
+}
+
 sub server_page {
 
 	my ($no_fork) = @_;
@@ -1790,9 +1958,12 @@ my $pretty_vector = unpack('b*', $rin);
 			if($no_fork) {
 				### Careful, returns after MaxRequests or terminate signal
 				$::Instance = {};
-				$handled++;
 #::logDebug("begin non-forked ::connection()");
-				connection();
+				send_ipc(sprintf ('lastused %s %s',$$,time))
+					if $Global::PIDcheck;
+				connection(++$handled);
+				send_ipc(sprintf ('lastused %s %s',$$,time))
+					if $Global::PIDcheck;
 #::logDebug("end non-forked ::connection()");
 				undef $::Instance;
 			}
@@ -1901,8 +2072,6 @@ my $pretty_vector = unpack('b*', $s_vector);
 				last;
 			}
 
-
-
 	  };
 
 	  last if $Signal_Terminate;
@@ -2018,8 +2187,13 @@ sub process_ipc {
 		close $fh;
 		$Num_servers--;
 	}
+	elsif ($thing =~ /^lastused (\d+) (\d+)/) {
+#::logDebug("Page pid $1 last used at $2");
+		$Page_pids{$1} = $2;
+	}
 	elsif ($thing =~ /^register page (\d+)/) {
-		$Page_pids{$1} = 1;
+		$Page_pids{$1} = time;
+		starting_pids('del',$1);
 #::logDebug("registered Page pid $1");
 		$Page_servers++;
 	}
@@ -2187,12 +2361,16 @@ sub server_both {
 		};
 		if($@) {
 			::logGlobal( {
-				level => 'warn' },
-				"SOAP enabled, but Vend::SOAP will not load: %s",
+				level => 'info' },
 				$@,
 			);
-		}
-		else {
+			::logGlobal( {
+				level => 'warn' },
+				"SOAP enabled, but Vend::SOAP failed to load."
+			);
+			print "SOAP enabled, but Vend::SOAP failed to load.\n";
+			$Global::SOAP = 0;
+		} else {
 			my @made;
 			my @unix_soap = grep m{/}, @{$Global::SOAP_Socket};
 			my @inet_soap = grep $_ !~ m{/}, @{$Global::SOAP_Socket};
@@ -2205,7 +2383,7 @@ sub server_both {
 			}
 			if(@inet_soap) {
 				push @made,
-					map_inet_socket(\$s_vector, \%s_vec_map, \%s_fh_map, @inet_soap);
+					map_inet_socket('SOAP', \$s_vector, \%s_vec_map, \%s_fh_map, @inet_soap);
 			}
 		}
 	}
@@ -2221,7 +2399,7 @@ sub server_both {
 				$Global::TcpHost,
 				);
 		my @made =
-			map_inet_socket(\$vector, \%vec_map, \%fh_map, keys %{$Global::TcpMap});
+			map_inet_socket('TCP', \$vector, \%vec_map, \%fh_map, keys %{$Global::TcpMap});
 		if (! scalar @made) {
 			my $msg;
 			if ($Global::Unix_Mode) {
@@ -2249,6 +2427,11 @@ sub server_both {
 
 	::logGlobal({ level => 'info' }, server_start_message() );
 
+	print server_start_message(
+		"Interchange server started in %s mode(s) (process id %s)\n",
+		1,
+	) unless $Vend::Quiet;
+
 	my $no_fork;
 	if($Global::Windows or $Global::DEBUG ) {
 		$no_fork = 1;
@@ -2267,7 +2450,7 @@ sub server_both {
 	if($Global::StartServers) {
 		$master_ipc = 1;
 		$p_vector = $vector ^ $ipc_vector;
-		start_page(1,$Global::PreFork);
+		start_page(1, $Global::PreFork, $Global::StartServers);
 	}
 
 	my $c = 0;
@@ -2466,29 +2649,103 @@ my $pretty_vector = unpack('b*', $rin);
 			);
 			kill 'TERM', @pids;
 		}
+		for(keys %Global::Catalog) {
+			::remove_catalog($_);
+		}
    	}
 
     return '';
 }
 
 sub touch_pid {
-	open(TEMPPID, ">>$Global::RunDir/pid.$$") 
+	my $temppid = gensym();
+	
+	open($temppid, ">>$Global::RunDir/pid.$$") 
 		or die "create PID file $$: $!\n";
-	lockfile(\*TEMPPID, 1, 0)
+	lockfile($temppid, 1, 0)
 		or die "PID $$ conflict: can't lock\n";
+	
+	if (@_) {
+		$temppid->autoflush(1);
+		print $temppid $_[0], "\n";
+	}
 }
 
 sub jobs_job {
 	my ($cat, @jobs) = @_;
+	my $parms;
+
+	if (ref($jobs[$#jobs]) eq 'HASH') {
+		$parms = pop(@jobs);
+	}
+	
 	for my $job (@jobs) {
-		Vend::Dispatch::run_in_catalog($cat, $job);
+		Vend::Dispatch::run_in_catalog($cat, $job, '', $parms);
+	}
+}
+
+sub flag_job {
+	my ($pid, $cat, $action, $token) = @_;
+
+	if ($action eq 'raise') {
+		if ($token =~ /^(\d+)$/) {
+			my $file = "flag.$cat.$1";
+			my $cwd = getcwd();
+		   
+			unless (open(FLAG, ">>$Global::RunDir/$file")) {
+				die "unable to create flag file $Global::RunDir/$file: $!\n";
+			}
+
+			unless (lockfile(\*FLAG, 1, 0)) {
+				die "unable to lock file $Global::RunDir/$file: $!\n";
+			}
+
+			unless (chdir($Global::RunDir)) {
+				die "unable to enter directory $Global::RunDir: $!\n";
+			}
+
+			unless (symlink($file, "flag.$pid")) {
+				chdir($cwd);
+				die "unable to create symlink for $file: $!\n";
+			}
+
+			chdir($cwd);
+		} else {
+			return undef;
+		}
+	} elsif ($action eq 'check') {
+		return if $token !~ /^(\d+)$/;
+
+		if (-f "$Global::RunDir/flag.$cat.$1") {
+			return 1;
+		} else {
+			return 0;
+		}
+	} elsif ($action eq 'furl') {
+		my $flagfile = readlink("$Global::RunDir/flag.$pid");
+
+		if (defined ($flagfile)) {
+			if ($flagfile =~ /^flag\.$cat\.(\d+)$/) {
+				unless (unlink("$Global::RunDir/$flagfile")) {
+					die "failed to remove flag file: $Global::RunDir/$flagfile: $!\n";
+				}
+			} else {
+				die "invalid flag file $flagfile\n";
+			}
+
+			unless (unlink("$Global::RunDir/flag.$pid")) {
+				die "failed to remove link to flag file: $Global::RunDir/flag.$pid: $!\n";
+			}
+		} else {
+			logGlobal({level => 'notice'}, "Readlink failed: $!\n");
+		}
 	}
 }
 
 sub run_jobs {
 	my ($cat, @jobs) = @_;
 
-#::logGlobal("Vend::Server::run_jobs: run jobs cat=cat jobs=@jobs");
+#::logGlobal("Vend::Server::run_jobs: run jobs cat=$cat job=@jobs");
 	my $pid;
 	if($Global::Foreground) {
 		$::Instance = {};
@@ -2517,7 +2774,7 @@ sub run_jobs {
 			reset_per_fork();
 			$::Instance = {};
 			eval { 
-				touch_pid() if $Global::PIDcheck;
+				touch_pid($cat) if $Global::PIDcheck;
 				&$Sig_inc;
 				jobs_job($cat, @jobs);
 			};
@@ -2717,10 +2974,6 @@ sub run_server {
 						);
                     exit 1;
                 }
-                print server_start_message(
-						"Interchange server started in %s mode(s) (process id %s)\n",
-						1,
-					 ) unless $Vend::Quiet;
 
                 setsid();
 
@@ -2739,6 +2992,26 @@ sub run_server {
     }
 }
 
+# Set the process name ($0) according to MV_DOLLAR_ZERO and a status indicator.
+sub set_process_name {
+    my $status = shift;
+    my $base = $Global::Variable->{MV_DOLLAR_ZERO};
+
+    # Setting MV_DOLLAR_ZERO to 1 should do the same thing as not setting it for
+    # backwards compatibility.
+    $base = 'interchange' if !$base or $base eq '1';
+
+    if (defined $status) {
+	$0 = "$base: $status";
+    }
+    else {
+	$0 = $base;
+    }
+
+    return;
+}
+
+
 1;
 __END__
 
