@@ -1,8 +1,8 @@
 # Vend::Ship - Interchange shipping code
 # 
-# $Id: Ship.pm,v 2.13.2.3 2006/09/20 12:33:42 kwalsh Exp $
+# $Id: Ship.pm,v 2.26 2007/08/09 13:40:54 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Ship;
 
@@ -68,7 +68,7 @@ my %Ship_remap = ( qw/
 sub do_error {
 	my $msg = errmsg(@_);
 	Vend::Tags->error({ name => 'shipping', set => $msg });
-	unless ($Vend::Cfg->{Limit}{no_ship_message}) {
+	unless ($::Limit->{no_ship_message}) {
 		$Vend::Session->{ship_message} ||= '';
 		$Vend::Session->{ship_message} .= $msg . ($msg =~ / $/ ? '' : ' ');
 	}
@@ -97,6 +97,44 @@ use vars qw/%Ship_handler/;
 		,
 );
 
+sub process_new_beginning {
+	my ($shipping, $record, $line) = @_;
+	my @new;
+	my $first;
+
+	$line ||= '';
+	if($line =~ /^[^\s:]+\t/) {
+		@new = split /\t/, $line;
+	}
+	elsif($line =~ /^(\w+)\s*:\s*(.*)/s) {
+		@new = ($1, $2, '', 0, 99999999, 0);
+		$first = 1;
+	}
+
+	$Vend::Cfg->{Shipping_desc}{$new[MODE]} ||= $new[DESC] if @new;
+
+	if (@$record) {
+		my $old_mode = $record->[MODE];
+		if(! ref($record->[OPT]) ) {
+			$record->[OPT] = string_to_ref($record->[OPT]);
+		}
+
+		if ($old_mode and ! $Vend::Cfg->{Shipping_hash}{$old_mode}) {
+			$record->[OPT]{description} ||= $Vend::Cfg->{Shipping_desc}{$old_mode};
+			$Vend::Cfg->{Shipping_hash}{$old_mode} = $record->[OPT];
+		}
+		else {
+			$record->[OPT]{description} ||= $record->[DESC];
+		}
+
+		push @$shipping, [ @$record ];
+	}
+
+	@$record = @new;
+
+	return $first;
+}
+
 sub read_shipping {
 	my ($file, $opt) = @_;
 	$opt = {} unless $opt;
@@ -167,8 +205,8 @@ sub read_shipping {
 		}
 	}
 	
-	$Vend::Cfg->{Shipping_desc} = {}
-		if ! $Vend::Cfg->{Shipping_desc};
+	$Vend::Cfg->{Shipping_desc} ||= {};
+
 	my %seen;
 	my $append = '00000';
 	my @line;
@@ -211,37 +249,13 @@ sub read_shipping {
 
 		next if ! /\S/ or /^\s*#/;
 		s/\s+$//;
-		if(/^[^\s:]+\t/) {
-			push (@shipping, [@line]) if @line;
-			@line = split(/\t/, $_);
-			$Vend::Cfg->{Shipping_desc}->{$line[0]} = $line[1]
-				unless $seen{$line[0]}++;
-			push @shipping, [@line];
-			@line = ();
-		}
-		elsif(/^(\w+)\s*:\s*(.*)/s) {
-			push (@shipping, [@line]) if @line;
-			@line = ($1, $2, 'quantity', 0, 999999999, 0);
-			$first = 1;
-			$Vend::Cfg->{Shipping_desc}->{$line[0]} = $line[1]
-				unless $seen{$line[0]}++;
-			next;
-		}
-		elsif(/^\s+min(?:imum)?\s+(\S+)/i) {
-			my $min = $1;
-			if ($first) {
-				undef $first;
-				$line[MIN] = $min;
-			}
-			else {
-				push @shipping, [ @line ];
-				$line[MIN] = $min;
-				if(ref $line[OPT]) {
-					my $ref = $line[OPT];
-					$line[OPT] = { %$ref };
-				}
 
-			}
+		if(/^[^\s:]+\t/ or /^\w+\s*:/s) {
+			## This along with process_new_beginning replaces
+			## two previous branches that had same code doing
+			## same thing.
+
+			$first = process_new_beginning(\@shipping, \@line, $_);
 		}
 		else {
 			no strict 'refs';
@@ -251,6 +265,19 @@ sub read_shipping {
 			$k = uc $k;
 			$k = $Ship_remap{$k}
 				if defined $Ship_remap{$k};
+
+			if ($k eq 'MIN') {
+				# Special case handling for minimum line.
+				if ($first) {
+					undef $first;
+				}
+				else {
+					# Push the record we have to this point.
+					my @lcopy = @line;
+					process_new_beginning(\@shipping, \@lcopy);
+				}
+			}
+
 			$Ship_handler{$k}->(\$v, \$k, \@line)
 				if defined $Ship_handler{$k};
 			eval {
@@ -271,8 +298,7 @@ sub read_shipping {
 		}
 	}
 
-	push @shipping, [ @line ]
-		if @line;
+	process_new_beginning(\@shipping, \@line);
 
 	if($waiting) {
 		logError(
@@ -370,13 +396,15 @@ sub read_shipping {
 				next;
 			}
 			my (@zone) = grep /\S/, split /[\r\n]+/, $ref->{zone_data};
-			shift @zone while @zone and $zone[0] !~ /^(Postal|Dest.*Z)/;
+			shift @zone while @zone and $zone[0] !~ /^(Postal|Dest.*Z|low)/;
 			if($zone[0] =~ /^Postal/) {
 				$zone[0] =~ s/,,/,/;
 				for(@zone[1 .. $#zone]) {
 					s/,/-/;
 				}
 			}
+			@zone = grep /\S/, @zone;
+			@zone = grep /^[^"]/, @zone;
 			if($zone[0] !~ /\t/) {
 				my $len = $ref->{str_length} || 3;
 				@zone = grep /\S/, @zone;
@@ -672,7 +700,8 @@ sub shipping {
 		my $value;
 		if ($use_modifier && defined $item->{$field}){
 		    $value = $item->{$field};
-		}else{
+		}
+		else{
 		    unless ($col_checked++ || column_exists $field){
 			logError("Custom shipping field '$field' doesn't exist. Returning 0");
 			$total = 0;
@@ -966,7 +995,7 @@ sub tag_shipping {
 	$Ship_its = 0;
 	if(! $mode) {
 		if($opt->{widget} || $opt->{label}) {
-			$mode = resolve_shipmode(undef, { possible => 1});
+			$mode = resolve_shipmode(undef, { no_set => $opt->{no_set}, possible => 1});
 		}
 		else {
 			$mode = $opt->{handling}
@@ -1120,7 +1149,21 @@ sub tag_ups {
 
 	my $rawzip = $zip;
 
-	$zip = substr($zip, 0, ($zref->{str_length} || 3));
+	my $country;
+	if($opt->{country_prefix}) {
+		$country = $::Values->{country} || '';
+		$country = uc $country;
+		$country =~ s/\W+//g;
+		$country =~ m{^\w\w$} 
+			or do {
+				logDebug('Country code not present with country_prefix');
+				return undef;
+			};
+		$zip = $country . ":" . $zip;
+	}
+	else {
+		$zip = substr($zip, 0, ($zref->{str_length} || 3));
+	}
 
 	@fieldnames = split /\t/, $zdata->[0];
 	for($i = 2; $i < @fieldnames; $i++) {
@@ -1146,8 +1189,17 @@ sub tag_ups {
 	}
 
 #::logDebug("tag_ups looking in zone data.");
+	my $zip_trimmed;
 	for(@{$zdata}[1..$#{$zdata}]) {
 		@data = split /\t/, $_;
+
+                unless($zip_trimmed) {
+			if ( $data[0] =~ m{^(([A-Z][A-Z]):)?(\w+)} and $2 eq $country ) {
+				$zip = substr($zip, 0, length($1.$3));
+				$zip_trimmed++;
+			}
+		}		
+
 		next unless ($zip ge $data[0] and $zip le $data[1]);
 		$zone = $data[$point];
 		$eas_zone = $data[$eas_point] if defined $eas_point;
@@ -1206,9 +1258,9 @@ sub tag_ups {
 
 sub tag_shipping_desc {
 	my $mode = 	shift;
+	my $key = shift || 'description';
 	$mode = $mode || $::Values->{mv_shipmode} || 'default';
-	return '' unless defined $Vend::Cfg->{Shipping_desc}{$mode};
-	return errmsg($Vend::Cfg->{Shipping_desc}{$mode});
+	return errmsg($Vend::Cfg->{Shipping_hash}{$mode}{$key});
 }
 
 =head1 NAME
