@@ -1,8 +1,8 @@
-# Vend::Ship - Interchange shipping code
+# Vend::Ship::Postal - Interchange shipping code
 # 
-# $Id: Postal.pm,v 1.4 2005/11/08 18:14:47 jon Exp $
+# $Id: Postal.pm,v 1.8 2007/08/09 13:40:56 pajamian Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Ship::Postal;
 
@@ -99,9 +99,9 @@ sub calculate {
 			my $w = $weight;
 			while($w > $modulo) {
 				$w -= $modulo;
-				$cost += tag_postal($type, $modulo, $country, $opt);
+				$cost += calculate($type, $modulo, $country, $opt);
 			}
-			$cost += tag_postal($type, $w, $country, $opt);
+			$cost += calculate($type, $w, $country, $opt);
 			return $cost;
 		}
 	}
