@@ -1,10 +1,10 @@
 # Vend::Swish - Search indexes with Swish-e
 #
-# $Id: Swish.pm,v 1.6 2005/11/08 18:14:45 jon Exp $
+# $Id: Swish.pm,v 1.11 2007/08/09 13:40:54 pajamian Exp $
 #
 # Adapted from Vend::Glimpse
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 2002 Mike Heins <mikeh@perusion.net>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,14 +19,14 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Swish;
 require Vend::Search;
 @ISA = qw(Vend::Search);
 
-$VERSION = substr(q$Revision: 1.6 $, 10);
+$VERSION = substr(q$Revision: 1.11 $, 10);
 use strict;
 
 sub array {
@@ -66,7 +66,7 @@ sub init {
 #::logDebug("initting Swish search, Swish=" . Vend::Util::uneval($Vend::Cfg->{Swish}));
 	$Vend::Cfg->{Swish} ||= {};
 	@{$s}{keys %Default} = (values %Default);
-	$s->{mv_base_directory}     = $Vend::Cfg->{Swish}{configfile} || undef,
+	$s->{mv_base_directory}     = undef,
 	$s->{mv_begin_string}       = [];
 	$s->{mv_all_chars}	        = [1];
 	$s->{mv_case}               = [];
@@ -114,8 +114,11 @@ sub search {
 	my(@specs);
 	my(@pats);
 
+	# map Swish-e auto properties to field names
 	my %fmap = qw/
 					code	swishreccount
+					description swishdescription
+					dbfile    swishdbfile
 					score	swishrank
 					url		swishdocpath
 					title	swishtitle
@@ -166,8 +169,8 @@ sub search {
 	push @cmd, "-c $s->{mv_base_directory}"
 			if $s->{mv_base_directory};
 
-	if(@{$s->{mv_search_file} || []} ) {
-		push @cmd, "-f " . join(" ", @{$s->{mv_search_file}} );
+	if (@searchfiles) {
+		push @cmd, "-f " . join(" ", @searchfiles);
 	}
 	
 	push @cmd, "-m $s->{mv_max_matches}" if $s->{mv_max_matches};
@@ -258,10 +261,16 @@ EOF
 			push @out, $return_sub->($_);
 		}
 	}
+	
+	if(scalar(@out) == 1 and $out[0][0] =~ s/^err:\s*(.*?)\s*$//)  {
+		# presumably search error signaled by Swish
+		@out = ();
 
-	if(scalar(@out) == 1 and $out[0][0] =~ s/^err\w*\W+//)  {
-		$s->{matches} = -1;
-		return $s->search_error($out[0][0]);
+		# don't consider missing matches as search error
+	    unless ($1 eq 'no results') {
+			$s->{matches} = -1;
+			return $s->search_error($1);
+		}
 	}
 
 	$s->{matches} = scalar(@out);
