@@ -1,8 +1,8 @@
 # Vend::Table::DBI - Access a table stored in an DBI/DBD database
 #
-# $Id: DBI.pm,v 2.68 2005/11/08 18:14:47 jon Exp $
+# $Id: DBI.pm,v 2.80 2007/08/14 13:42:08 kwalsh Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,11 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Table::DBI;
-$VERSION = substr(q$Revision: 2.68 $, 10);
+$VERSION = substr(q$Revision: 2.80 $, 10);
 
 use strict;
 no warnings qw(uninitialized numeric);
@@ -176,6 +176,9 @@ my %known_capability = (
 		Pg => 'CREATE _UNIQUE_ INDEX $TABLE$_$COLUMN$ ON _TABLE_ (_COLUMN_)',
 		default => 'CREATE _UNIQUE_ INDEX $TABLE$_$COLUMN$ ON _TABLE_ (_COLUMN_)',
 	},
+	LIST_FIELDS_QUERY => { 
+		mysql => 'SELECT * FROM `_TABLE_` WHERE 2 = 1',
+	},
 	SEQUENCE_CREATE	 => { 
 		Oracle => "CREATE SEQUENCE _SEQUENCE_NAME_",
 		Pg => "CREATE SEQUENCE _SEQUENCE_NAME_",
@@ -183,6 +186,14 @@ my %known_capability = (
 	HAS_TABLE_TYPE	 => { 
 		mysql => 1,
 	},
+	TABLE_COMMENT_SQL => { 
+		mysql  => 'COMMENT=_COMMENT_',
+		Pg     => 'COMMENT ON _TABLE_ IS _COMMENT_',
+		Oracle => 'COMMENT ON _TABLE_ IS _COMMENT_',
+	},
+	TABLE_COMMENT_DURING_CREATE => { 
+		mysql => 1,
+	},
 	SEQUENCE_QUERY	 => { 
 		Oracle => "SELECT _SEQUENCE_NAME_.nextval FROM dual",
 		Pg => "SELECT nextval('_SEQUENCE_NAME_')",
@@ -292,8 +303,28 @@ sub create_sql {
 	my $query = "create table $tablename ( \n";
 	$query .= join ",\n", @cols;
 	$query .= "\n)\n";
-	if($config->{HAS_TABLE_TYPE} and $config->{TABLE_TYPE} ) {
-		$query =~ s/\s*$/ type=$config->{TABLE_TYPE}\n/;
+
+	if ($config->{TABLE_TYPE} && $config->{HAS_TABLE_TYPE} ) {
+		$query =~ s/\s*$/ TYPE=$config->{TABLE_TYPE}\n/;
+	}
+
+	if ($config->{TABLE_COMMENT} && $config->{TABLE_COMMENT_SQL}) {
+		my $comment = $config->{TABLE_COMMENT};
+		$comment =~ s/^\s*(["'])\s*(.*?)\s*\1\s*$/$2/;
+		$comment =~ s/'/''/g;
+
+		my $template = $config->{TABLE_COMMENT_SQL};
+		$template =~ s/\b_COMMENT_\b/'$comment'/;
+
+		if ($config->{TABLE_COMMENT_DURING_CREATE}) {
+			$query =~ s/\s*$/ $template\n/;
+		}
+		else {
+			$template =~ s/\b_TABLE_\b/$tablename/;
+
+			$config->{POSTCREATE} ||= [];
+			push(@{$config->{POSTCREATE}},$template);
+		}
 	}
 	return $query;
 }
@@ -325,7 +356,7 @@ sub create {
 					|| $DBI::errstr
 					|| "unknown error. Driver '$dname' installed?";
 		}
-		die ::errmsg("connect failed (create) -- %s\n",$msg);
+		die ::errmsg("connect to %s failed (create) -- %s\n", $call[0], $msg);
 	}
 
 	# Allow multiple tables in different DBs to have same local name
@@ -1227,6 +1258,10 @@ sub set_slice {
 
 
 	if ( defined $tkey and $s->record_exists($key) ) {
+		unless (@$fary) {
+			# as there are no data columns, we can safely skip the update
+			return $key;
+		}
 		my $fstring = join ",", map { "$_=?" } @$fary;
 		$sql = "update $s->[$TABLE] SET $fstring WHERE $s->[$KEY] = $tkey";
 	}
@@ -1258,9 +1293,12 @@ sub set_slice {
 		my $rc = $sth->execute(@$vary)
 			or die ::errmsg("execute %s: %s", $sql, $DBI::errstr);
 
-		$val	= $s->[$CONFIG]->{AUTO_SEQUENCE}
-				? $s->last_sequence_value($key)
-				: $key;
+		if (length $key) {
+		    $val = $key;
+		}
+		else {
+		    $val = $s->last_sequence_value;
+		}
 	};
 
 #::logDebug("set_slice key: $val");
@@ -1290,13 +1328,6 @@ sub set_row {
 	$s->filter(\@fields, $s->[$CONFIG]{COLUMN_INDEX}, $s->[$CONFIG]{FILTER_TO})
 		if $cfg->{FILTER_TO};
 
-	if ($cfg->{PREFER_NULL}) {
-		for (keys %{$cfg->{PREFER_NULL}}) {
-			my $i = $cfg->{COLUMN_INDEX}{$_};
-			undef $fields[$i] if $fields[$i] eq '';
-		}
-	}
-
 	my $val;
 
 	if(scalar @fields == 1) {
@@ -1350,6 +1381,13 @@ sub set_row {
 		}
 		return $fields[0];
 	}
+	
+	if ($cfg->{PREFER_NULL}) {
+		for (keys %{$cfg->{PREFER_NULL}}) {
+			my $i = $cfg->{COLUMN_INDEX}{$_};
+			undef $fields[$i] if $fields[$i] eq '';
+		}
+	}
 
 	if(! length($fields[$ki]) ) {
 		$fields[$ki] = $s->autonumber();
@@ -1390,12 +1428,15 @@ sub set_row {
 #::logDebug("set_row fields='" . join(',', @fields) . "'" );
     $s->bind_entire_row($cfg->{_Insert_h}, @fields);
 
-	my $rc = $cfg->{_Insert_h}->execute()
-		or die $s->log_error("%s error on key '%s': $DBI::errstr", 'set_row', $fields[$ki], $DBI::errstr);
+    my $rc = $cfg->{_Insert_h}->execute()
+	or die $s->log_error("%s error on key '%s': $DBI::errstr", 'set_row', $fields[$ki], $DBI::errstr);
 
-	$val	= $cfg->{AUTO_SEQUENCE}
-			?  $s->last_sequence_value($fields[$ki])
-			: $fields[$ki];
+    if (length $fields[$ki]) {
+	$val = $fields[$ki];
+    }
+    else {
+	$val = $s->last_sequence_value;
+    }
 
 #::logDebug("set_row rc=$rc key=$val");
 	return $val;
@@ -1743,7 +1784,8 @@ sub list_fields {
 	my($db, $name, $config) = @_;
 	my @fld;
 
-	my $q = "SELECT * FROM $name WHERE 2 = 1";
+	my $q = $config->{LIST_FIELDS_QUERY} || "SELECT * FROM _TABLE_ WHERE 2 = 1";
+	$q =~ s/\b_TABLE_\b/$name/g;
 
 	my $sth = $db->prepare($q)
 		or die ::errmsg("%s prepare on %s: %s", 'list_fields', $name, $DBI::errstr);
@@ -1848,7 +1890,6 @@ sub each_nokey {
             or die $s->log_error('prepare');
 		$query->execute()
             or die $s->log_error('execute');
-		my $idx = $s->[$CONFIG]{KEY_INDEX};
 		$each = sub {
 			my $ref = $query->fetchrow_arrayref()
 				or return undef;
@@ -1983,8 +2024,8 @@ sub query {
 			my $newdb;
 			eval {
 				$trytab = Vend::Scan::sql_statement($query, { table_only => 1 } );
-				$newdb = Vend::Data::database_exists_ref($trytab);
-				if($newdb->config('name') eq $s->config('name')) {
+				if(($newdb = Vend::Data::database_exists_ref($trytab))
+				   && $newdb->config('name') eq $s->config('name')) {
 					die $origmsg;
 				}
 			};
@@ -2130,8 +2171,13 @@ eval {
 sub auto_config {
 	my $string = shift;
 	my ($dsn, $user, $pass, $catalog, $schema, $name, $type) = Text::ParseWords::shellwords($string);
-	my $handle = DBI->connect($dsn, $user, $pass)
-		or ::logDebug(::errmsg("DatabaseAuto DSN '%s' does not connect.", $dsn));
+	my $handle = DBI->connect($dsn, $user, $pass);
+
+	unless ($handle) {
+		::Vend::Config::config_error(::errmsg("DatabaseAuto DSN '%s' does not connect: %s", $dsn, $DBI::errstr));
+		return;
+	}
+	
 	my @tabs;
 	my @out;
 	my $sth;
