@@ -1,8 +1,8 @@
 # Vend::UserDB - Interchange user database functions
 #
-# $Id: UserDB.pm,v 2.41 2005/11/08 18:14:45 jon Exp $
+# $Id: UserDB.pm,v 2.55 2007/08/11 13:54:14 mheins Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 package Vend::UserDB;
 
-$VERSION = substr(q$Revision: 2.41 $, 10);
+$VERSION = substr(q$Revision: 2.55 $, 10);
 
 use vars qw!
 	$VERSION
@@ -272,6 +272,7 @@ sub new {
 			LAST   		=> '',
 			USERMINLEN	=> $options{userminlen}	|| 2,
 			PASSMINLEN	=> $options{passminlen}	|| 4,
+			VALIDCHARS	=> $options{validchars} ? ('[' . $options{validchars} . ']') : $USERNAME_GOOD_CHARS,
 			CRYPT  		=> defined $options{'crypt'}
 							? $options{'crypt'}
 							: ! $::Variable->{MV_NO_CRYPT},
@@ -530,6 +531,10 @@ sub set_db {
 		$ignore{$_} = 1;
 	}
 
+	if($self->{OPTIONS}{username_email}) {
+		$ignore{$self->{OPTIONS}{username_email_field} || 'email'} = 1;
+	}
+
 	for(values %{$self->{LOCATION}}) {
 		$ignore{$_} = 1;
 	}
@@ -568,22 +573,52 @@ sub clear_values {
 
 	@fields = @{ $self->{DB_FIELDS} } unless @fields;
 
+	my %constant;
 	my %scratch;
+	my %session_hash;
+
+	if($self->{OPTIONS}->{constant}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{constant} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$constant{$k} = $v;
+		}
+	}
 
 	if($self->{OPTIONS}->{scratch}) {
-		my (@s) = split /[\s,]+/, $self->{OPTIONS}{scratch} ;
-		@scratch{@s} = @s;
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{scratch} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$scratch{$k} = $v;
+		}
+	}
+
+	if($self->{OPTIONS}->{session_hash}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{session_hash} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$session_hash{$k} = $v;
+		}
 	}
 
 	for(@fields) {
-		if($scratch{$_}) {
-			if (exists $Vend::Cfg->{ScratchDefault}->{$_}) {
-				$::Scratch->{$_} = $Vend::Cfg->{ScratchDefault}->{$_};
+		if(my $s = $scratch{$_}) {
+			if (exists $Vend::Cfg->{ScratchDefault}->{$s}) {
+				$::Scratch->{$s} = $Vend::Cfg->{ScratchDefault}->{$s};
 			}
 			else {
-				delete $::Scratch->{$_};
+				delete $::Scratch->{$s};
 			}
 		}
+		elsif($constant{$_}) {
+			delete $Vend::Session->{constant}{$constant{$_}};
+		}
+		elsif($session_hash{$_}) {
+			delete $Vend::Session->{$session_hash{$_}};
+		}
 		else {
 			if (exists $Vend::Cfg->{ValuesDefault}->{$_}) {
 				$::Values->{$_} = $Vend::Cfg->{ValuesDefault}->{$_};
@@ -603,9 +638,14 @@ sub get_values {
 
 	$valref = $::Values unless ref($valref);
 	$scratchref = $::Scratch unless ref($scratchref);
+	my $constref = $Vend::Session->{constant} ||= {};
 
 	my @fields = @{ $self->{DB_FIELDS} };
 
+	if($self->{OPTIONS}{username_email}) {
+		push @fields, $self->{OPTIONS}{username_email_field} || 'email';
+	}
+
 	my $db = $self->{DB}
 		or die errmsg("No user database found.");
 
@@ -616,6 +656,8 @@ sub get_values {
 
 	my %ignore;
 	my %scratch;
+	my %constant;
+	my %session_hash;
 
 	for(values %{$self->{LOCATION}}) {
 		$ignore{$_} = 1;
@@ -627,10 +669,34 @@ sub get_values {
 		push @fields, keys %outboard;
 	}
 
+	if($self->{OPTIONS}->{constant}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{constant} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$constant{$k} = $v;
+		}
+#::logDebug("constant ones: " . join " ", @s);
+	}
+
+	if($self->{OPTIONS}->{session_hash}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{session_hash} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$session_hash{$k} = $v;
+		}
+#::logDebug("session_hash ones: " . join " ", @s);
+	}
+
 	if($self->{OPTIONS}->{scratch}) {
-		my (@s) = split /[\s,]+/, $self->{OPTIONS}{scratch} ;
-		@scratch{@s} = @s;
-#::logError("scratch ones: " . join " ", @s);
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{scratch} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$scratch{$k} = $v;
+		}
+#::logDebug("scratch ones: " . join " ", @s);
 	}
 
 	my @needed;
@@ -665,11 +731,20 @@ sub get_values {
 			$val = $row->{$_};
 		}
 
-		if($scratch{$_}) {
-			$::Scratch->{$_} = $val;
+		my $k;
+		if($k = $scratch{$_}) {
+			$scratchref->{$k} = $val;
 			next;
 		}
-		$::Values->{$_} = $val;
+		elsif($k = $constant{$_}) {
+			$constref->{$k} = $val;
+			next;
+		}
+		elsif($k = $session_hash{$_}) {
+			$Vend::Session->{$k} = string_to_ref($val) || {};
+			next;
+		}
+		$valref->{$_} = $val;
 
 	}
 
@@ -703,10 +778,34 @@ sub set_values {
 		return undef;
 	}
 	my %scratch;
+	my %constant;
+	my %session_hash;
 
 	if($self->{OPTIONS}->{scratch}) {
-		my (@s) = split /[\s,]+/, $self->{OPTIONS}{scratch} ;
-		@scratch{@s} = @s;
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{scratch} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$scratch{$k} = $v;
+		}
+	}
+
+	if($self->{OPTIONS}->{constant}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{constant} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$constant{$k} = $v;
+		}
+	}
+
+	if($self->{OPTIONS}->{session_hash}) {
+		my (@s) = grep /\w/, split /[\s,]+/, $self->{OPTIONS}{session_hash} ;
+		for(@s) {
+			my ($k, $v) = split /=/, $_;
+			$v ||= $k;
+			$session_hash{$k} = $v;
+		}
 	}
 
 	my $val;
@@ -742,9 +841,17 @@ sub set_values {
 	for( @fields ) {
 #::logDebug("set_values saving $_ as $valref->{$_}\n");
 		my $val;
-		if ($scratch{$_}) {
-			$val = $scratchref->{$_}
-				if defined $scratchref->{$_};	
+		my $k;
+		if ($k = $scratch{$_}) {
+			$val = $scratchref->{$k}
+				if defined $scratchref->{$k};	
+		}
+		elsif ($constant{$_}) {
+			# we never store constants
+			next;
+		}
+		elsif ($k = $session_hash{$_}) {
+			$val = uneval_it($Vend::Session->{$k});
 		}
 		else {
 			$val = $valref->{$_}
@@ -776,6 +883,8 @@ sub set_values {
 		push @bvals, shift @extra;
 	}
 
+#::logDebug("bfields=" . ::uneval(\@bfields));
+#::logDebug("bvals=" . ::uneval(\@bvals));
 	if(@bfields) {
 		$db->set_slice($user, \@bfields, \@bvals);
 	}
@@ -990,7 +1099,7 @@ sub get_hash {
 		$s = $self->{DB}->field( $self->{USERNAME}, $field_name);
 
 		if($s) {
-			$self->{$name} = $ready->reval($s);
+			$self->{$name} = string_to_ref($s);
 			die errmsg("Bad structure in %s: %s", $field_name, $@) if $@;
 		}
 		else {
@@ -1060,6 +1169,13 @@ sub login {
 				if ! $self->{PASSWORD};
 		}
 
+		if ($self->{VALIDCHARS} !~ / /) {
+			# If space isn't a valid character in usernames,
+			# be nice and strip leading and trailing whitespace.
+			$self->{USERNAME} =~ s/^\s+//;
+			$self->{USERNAME} =~ s/\s+$//;
+		}
+
 		if ($self->{OPTIONS}{ignore_case}) {
 			$self->{PASSWORD} = lc $self->{PASSWORD};
 			$self->{USERNAME} = lc $self->{USERNAME};
@@ -1081,7 +1197,7 @@ sub login {
 		}
 
 		# Username must contain only valid characters
-		if ($self->{USERNAME} !~ m{^$USERNAME_GOOD_CHARS+$}) {
+		if ($self->{USERNAME} !~ m{^$self->{VALIDCHARS}+$}) {
 			$self->log_either(errmsg("Denied attempted login for user name '%s' with illegal characters",
 				$self->{USERNAME}));
 			die $stock_error, "\n";
@@ -1499,6 +1615,9 @@ sub new_account {
 		die errmsg("Sorry, reserved user name.") . "\n"
 			if $self->{OPTIONS}{username_mask} 
 				and $self->{USERNAME} =~ m!$self->{OPTIONS}{username_mask}!;
+		die errmsg("Sorry, user name must be an email address.") . "\n"
+			if $self->{OPTIONS}{username_email} 
+				and $self->{USERNAME} !~ m!^[[:alnum:]]([.]?([[:alnum:]._-]+)*)?@([[:alnum:]\-_]+\.)+[a-zA-Z]{2,4}$!;
 		die errmsg("Must enter at least %s characters for password.",
 			$self->{PASSMINLEN}) . "\n"
 			if length($self->{PASSWORD}) < $self->{PASSMINLEN};
@@ -1532,11 +1651,17 @@ sub new_account {
 		}
 		die errmsg("Can't have '%s' as username; it contains illegal characters.",
 			$self->{USERNAME}) . "\n"
-			if $self->{USERNAME} !~ m{^$USERNAME_GOOD_CHARS+$};
+			if $self->{USERNAME} !~ m{^$self->{VALIDCHARS}+$};
 		die errmsg("Must have at least %s characters in username.",
 			$self->{USERMINLEN}) . "\n"
 			if length($self->{USERNAME}) < $self->{USERMINLEN};
 
+		if($self->{OPTIONS}{captcha}) {
+			my $status = Vend::Tags->captcha( { function => 'check' });
+			die errmsg("Must input captcha code correctly.\n") 
+				unless $status;
+		}
+
 		# Here we put the username in a non-primary key field, checking
 		# for existence
 		my $foreign = $self->{OPTIONS}{indirect_login};
@@ -1574,18 +1699,30 @@ sub new_account {
 						$self->{LOCATION}{PASSWORD},
 						$pw,
 						);
+
 		die errmsg("Database access error.") . "\n" unless defined $pass;
 
+		if($self->{OPTIONS}{username_email}) {
+			my $field_name = $self->{OPTIONS}{username_email_field} || 'email';
+			$::Values->{$field_name} ||= $self->{USERNAME};
+			$udb->set_field(
+						$self->{USERNAME},
+						$field_name,
+						$self->{USERNAME},
+						)
+				 or die errmsg("Database access error: %s", $udb->errstr) . "\n";
+		}
+
 		if($options{no_login}) {
 			$Vend::Session->{auto_created_user} = $self->{USERNAME};
 		}
 		else {
+			$self->set_values() unless $self->{OPTIONS}{no_set};
 			$self->{USERNAME} = $foreign if $foreign;
 			username_cookies($self->{USERNAME}, $pw) 
 				if $Vend::Cfg->{CookieLogin};
 
 			$self->log('new account') if $options{'log'};
-			$self->set_values() unless $self->{OPTIONS}{no_set};
 			$self->login()
 				or die errmsg(
 							"Cannot log in after new account creation: %s",
@@ -1658,7 +1795,7 @@ sub get_cart {
 		die errmsg("no saved carts.") . "\n" unless $s;
 
 		my @carts = split /\0/, $from;
-		my $d = $ready->reval($s);
+		my $d = string_to_ref($s);
 #::logDebug ("saved carts=" . ::uneval_it($d));
 
 		die errmsg("eval failed?")				unless ref $d;
@@ -1800,7 +1937,6 @@ sub userdb {
 			if( $Vend::ReadOnlyCfg->{AdminUserDB}{$user->{PROFILE}} ) {
 				$Vend::admin = 1;
 			}
-			undef $Vend::Cookie;
 			::update_user();
 		}
 	}
@@ -1814,7 +1950,6 @@ sub userdb {
 		if($status and ! $options{no_login}) {
 			$Vend::Session->{logged_in} = 1;
 			$Vend::Session->{username} = $user->{USERNAME};
-			undef $Vend::Cookie;
 		}
 	}
 	elsif($function eq 'logout') {
