@@ -1,8 +1,8 @@
 # Vend::Util - Interchange utility functions
 #
-# $Id: Util.pm,v 2.87.2.2 2006/08/30 18:56:22 mheins Exp $
+# $Id: Util.pm,v 2.107 2007/07/24 02:50:38 mheins Exp $
 # 
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -20,8 +20,8 @@
 #
 # You should have received a copy of the GNU General Public
 # License along with this program; if not, write to the Free
-# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-# MA  02111-1307  USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA  02110-1301  USA.
 
 package Vend::Util;
 require Exporter;
@@ -69,6 +69,8 @@ require Exporter;
 	show_times
 	string_to_ref
 	tag_nitems
+	timecard_stamp
+	timecard_read
 	uneval
 	uneval_it
 	uneval_fast
@@ -88,7 +90,7 @@ use Safe;
 use Vend::File;
 use subs qw(logError logGlobal);
 use vars qw($VERSION @EXPORT @EXPORT_OK);
-$VERSION = substr(q$Revision: 2.87.2.2 $, 10);
+$VERSION = substr(q$Revision: 2.107 $, 10);
 
 my $Eval_routine;
 my $Eval_routine_file;
@@ -115,7 +117,7 @@ $ESCAPE_CHARS::ok_in_url =
 	;
 
 ## This is a character class for HTML::Entities
-$ESCAPE_CHARS::std = "^\n\t !\#\$%\'-;=?-Z\\\]-~";
+$ESCAPE_CHARS::std = qq{^\n\t\X !\#\$%\'-;=?-Z\\\]-~};
 
 ## Some standard error templates
 
@@ -162,7 +164,7 @@ sub escape_chars {
     my($c, $r);
 
     $r = '';
-    foreach $c (split(//, $in)) {
+    foreach $c (split(m{}, $in)) {
 		$r .= $ESCAPE_CHARS::translate[ord($c)];
     }
 
@@ -179,7 +181,7 @@ sub escape_chars_url {
     my($c, $r);
 
     $r = '';
-    foreach $c (split(//, $in)) {
+    foreach $c (split(m{}, $in)) {
 		$r .= $ESCAPE_CHARS::translate_url[ord($c)];
     }
 
@@ -264,7 +266,7 @@ sub round_to_frac_digits {
 	$num =~ /^(\d*)(?:\.(\d+))?$/
 		or return $num;
 	my $int = $1;
-	@frac = split(//, ($2 || 0));
+	@frac = split(m{}, ($2 || 0));
 	local($^W) = 0;
 	my $frac = join "", @frac[0 .. $digits - 1];
 	if($frac[$digits] > 4) {
@@ -350,7 +352,7 @@ sub picture_format {
 	my $len = length($1);
 	$amount = sprintf('%.' . $len . 'f', $amount);
 	$amount =~ tr/0-9//cd;
-	my (@dig) = split //, $amount;
+	my (@dig) = split m{}, $amount;
 	$pic =~ s/#/pop(@dig)/eg;
 	$pic =~ s/\Q$sep\E+(?!\d)//;
 	$pic =~ s/\d/*/g if @dig;
@@ -417,12 +419,17 @@ sub setlocale {
     }
 
     if ($currency) {
-        my $curr = $Vend::Cfg->{Locale_repository}{$currency};
+        my $curr = $Vend::Cfg->{Currency_repository}{$currency};
 
         for(@Vend::Config::Locale_directives_currency) {
             $Vend::Cfg->{$_} = $curr->{$_}
                 if defined $curr->{$_};
         }
+
+        for(@Vend::Config::Locale_keys_currency) {
+            $Vend::Cfg->{Locale}{$_} = $curr->{$_}
+                if defined $curr->{$_};
+        }
     }
 
 	if(my $ref = $Vend::Cfg->{CodeDef}{LocaleChange}) {
@@ -458,10 +465,11 @@ sub setlocale {
 sub currency {
 	my($amount, $noformat, $convert, $opt) = @_;
 	$opt = {} unless $opt;
+	$convert ||= $opt->{convert};
 
 	my $pd = $Vend::Cfg->{PriceDivide};
 	if($opt->{locale}) {
-		$convert = 1;
+		$convert = 1 unless length($convert);
 		$pd = $Vend::Cfg->{Locale_repository}{$opt->{locale}}{PriceDivide};
 	}
 
@@ -469,6 +477,17 @@ sub currency {
 		$amount = $amount / $pd;
 	}
 
+	my $hash;
+	if(
+		$noformat =~ /\w+=\w\w/
+			and
+		ref($hash = get_option_hash($noformat)) eq 'HASH'
+	)
+	{
+		$opt->{display} ||= $hash->{display};
+		$noformat = $opt->{noformat} = $hash->{noformat};
+	}
+
 	return $amount if $noformat;
 	my $sep;
 	my $dec;
@@ -496,16 +515,41 @@ sub currency {
 		$dec = $loc->{mon_decimal_point} || $loc->{decimal_point} || '.';
 		return picture_format($amount, $loc->{price_picture}, $sep, $dec)
 			if defined $loc->{price_picture};
-		$fmt = "%." . $loc->{frac_digits} .  "f";
+		if (defined $loc->{frac_digits}) {
+			$fmt = "%." . $loc->{frac_digits} .  "f";
+		} else {
+			$fmt = "%.2f";
+		}
 		my $cs;
-		if($cs = ($loc->{currency_symbol} ||$loc->{currency_symbol} || '') ) {
-			if($loc->{p_cs_precedes}) {
+		my $display = lc($opt->{display}) || 'symbol';
+		my $sep_by_space = $loc->{p_sep_by_space};
+		my $cs_precedes = $loc->{p_cs_precedes};
+
+		if( $loc->{int_currency_symbol} && $display eq 'text' ) {
+			$cs = $loc->{int_currency_symbol};
+			$cs_precedes = 1;
+
+			if (length($cs) > 3 || $cs =~ /\W$/) {
+				$sep_by_space = 0;
+			}
+			else {
+				$sep_by_space = 1;
+			}
+		}
+		elsif ( $display eq 'none' ) {
+			$cs = '';
+		}
+		elsif ( $display eq 'symbol' ) {
+			$cs = $loc->{currency_symbol} || '';
+		}
+		if($cs) {
+			if ($cs_precedes) {
 				$precede = $cs;
-				$precede = "$precede " if $loc->{p_sep_by_space};
+				$precede = "$precede " if $sep_by_space;
 			}
 			else {
 				$succede = $cs;
-				$succede = " $succede" if $loc->{p_sep_by_space};
+				$succede = " $succede" if $sep_by_space;
 			}
 		}
 	}
@@ -709,6 +753,11 @@ sub logData {
 		close(MVLOGDATA) or die "close\n";
     };
     if ($@) {
+
+		if($::Limit->{logdata_error_length} > 0) {
+			$msg = substr($msg, 0, $::Limit->{logdata_error_length});
+		}
+
 		logError ("Could not %s log file '%s': %s\nto log this data:\n%s",
 				$@,
 				$file,
@@ -1078,6 +1127,8 @@ sub readin {
 		my $field = $Vend::Cfg->{PageTableMap}{page_text};
 		foreach my $t (@{$Vend::Cfg->{PageTables}}) {
 			my $db = Vend::Data::database_exists_ref($t);
+			next unless $db;
+
 			if($teleport) {
 				$file = teleport_name($file, $teleport, $t);
 			}
@@ -1582,8 +1633,38 @@ sub find_special_page {
 # Log the error MSG to the error file.
 
 sub logDebug {
-	return unless $Global::DebugFile;
-	print caller() . ':debug: ', errmsg(@_), "\n";
+    return unless $Global::DebugFile;
+	if(my $re = $Vend::Cfg->{DebugHost}) {
+		return unless
+			 Net::IP::Match::Regexp::match_ip($CGI::remote_addr, $re);
+	}
+
+	if(my $sub = $Vend::Cfg->{SpecialSub}{debug_qualify}) {
+		return unless $sub->();
+	}
+
+    if(my $tpl = $Global::DebugTemplate) {
+        my %debug; 
+		$tpl = POSIX::strftime($tpl, localtime());
+		$tpl =~ s/\s*$/\n/;
+		$debug{page} = $Global::Variable->{MV_PAGE};
+		$debug{tag} = $Vend::CurrentTag;
+		$debug{host} = $CGI::host || $CGI::remote_addr;
+		$debug{remote_addr} = $CGI::remote_addr;
+		$debug{catalog} = $Vend::Catalog;
+        if($tpl =~ /\{caller\d+\}/i) {
+            my @caller = caller();
+            for(my $i = 0; $i < @caller; $i++) {
+                $debug{"caller$i"} = $caller[$i];
+            }
+        }
+		$debug{message} = errmsg(@_);
+
+        print Vend::Interpolate::tag_attr_list($tpl, \%debug, 1);
+    }
+    else {
+        print caller() . ":debug: ", errmsg(@_), "\n";
+    }
 }
 
 sub errmsg {
@@ -1924,7 +2005,7 @@ sub send_mail {
 #::logDebug("using $using");
 		undef $none;
 
-		my $smtp = Net::SMTP->new($mhost, Debug => $Global::Variable->{DEBUG}, Hello => $helo);
+		my $smtp = Net::SMTP->new($mhost, Debug => $Global::Variable->{DEBUG}, Hello => $helo) or last SMTP;
 #::logDebug("smtp object $smtp");
 
 		my $from = $::Variable->{MV_MAILFROM}
@@ -2115,6 +2196,56 @@ sub codedef_options {
 	return \@out;
 }
 
+
+# Adds a timestamp to the end of a binary timecard file. You can specify the timestamp
+# as the second arg (unixtime) or just leave it out (or undefined) and it will be set
+# to the current time.
+sub timecard_stamp {
+	my ($filename,$timestamp) = @_;
+	$timestamp ||= time;
+
+	open(FH, '>>', $filename) or die "Can't open $filename for append: $!";
+	lockfile(\*FH, 1, 1);
+	binmode FH;
+	print FH pack('N',time);
+	unlockfile(\*FH);
+	close FH;
+}
+
+
+# Reads a timestamp from a binary timecard file.  If $index is negative indexes back from
+# the end of the file, otherwise indexes from the front of the file so that 0 is the first
+# (oldest) timestamp and -1 the last (most recent). Returns the timestamp or undefined if
+# the file doesn't exist or the index falls outside of the bounds of the timecard file.
+sub timecard_read {
+	my ($filename,$index) = @_;
+	$index *= 4;
+	my $limit = $index >= 0 ? $index + 4 : $index * -1;
+
+	if (-f $filename && (stat(_))[7] % 4) {
+	    # The file is corrupt, delete it and start over.
+	    ::logError("Counter file $filename found to be corrupt, deleting.");
+	    unlink($filename);
+	    return;
+	}
+	return unless (-f _ && (stat(_))[7] > $limit);
+
+	# The file exists and is big enough to cover the $index. Seek to the $index
+	# and return the timestamp from that position.
+
+	open (FH, '<', $filename) or die "Can't open $filename for read: $!";
+	lockfile(\*FH, 0, 1);
+	binmode FH;
+	seek(FH, $index, $index >= 0 ? 0 : 2) or die "Can't seek $filename to $index: $!";
+	my $rtime;
+	read(FH,$rtime,4) or die "Can't read from $filename: $!";
+	unlockfile(\*FH);
+	close FH;
+
+	return unpack('N',$rtime);
+}
+
+
 ### Provide stubs for former Vend::Util functions relocated to Vend::File
 *canonpath = \&Vend::File::canonpath;
 *catdir = \&Vend::File::catdir;
